Package: snakecase
Version: 0.11.1
Date: 2023-08-27
Title: Convert Strings into any Case
Description: A consistent, flexible and easy to use tool to parse and convert strings into cases like snake or camel among others.
Authors@R: c(
    person("Malte", "Grosser", , "malte.grosser@gmail.com", role = c("aut", "cre")))
Maintainer: Malte Grosser <malte.grosser@gmail.com>
Depends: R (>= 3.2)
Imports: stringr, stringi
Suggests: testthat, covr, tibble, purrrlyr, knitr, rmarkdown, magrittr
URL: https://github.com/Tazinho/snakecase
BugReports: https://github.com/Tazinho/snakecase/issues
Encoding: UTF-8
License: GPL-3
RoxygenNote: 6.1.1
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2023-08-27 20:30:20 UTC; malte
Author: Malte Grosser [aut, cre]
Repository: CRAN
Date/Publication: 2023-08-27 22:50:09 UTC
Built: R 4.6.0; ; 2026-04-27 21:49:57 UTC; unix

# Generated by roxygen2: do not edit by hand

export(to_any_case)
export(to_lower_camel_case)
export(to_lower_upper_case)
export(to_mixed_case)
export(to_parsed_case)
export(to_random_case)
export(to_screaming_snake_case)
export(to_sentence_case)
export(to_snake_case)
export(to_swap_case)
export(to_title_case)
export(to_upper_camel_case)
export(to_upper_lower_case)

# cases for examples
cases <-
  tibble::tribble(
    ~nr, ~ examples                  ,~parsed_case                    , ~snake_case                  , ~small_camel_case            , ~big_camel_case              , ~screaming_snake_case ,
    #--|-----------------------------,--------------------------------|------------------------------|------------------------------|------------------------------|-----------------------|
    1  , NA                          , NA                             , NA                             , NA                           , NA                           , NA,
    2  , "snake_case"                , "snake_case"                   , "snake_case"                   , "snakeCase"                  , "SnakeCase"                  , "SNAKE_CASE",
    3  , "snakeCase"                 , "snake_Case"                   , "snake_case"                   , "snakeCase"                  , "SnakeCase"                  , "SNAKE_CASE",
    4  , "SnakeCase"                 , "Snake_Case"                   , "snake_case"                   , "snakeCase"                  , "SnakeCase"                  , "SNAKE_CASE",
    5  , "_"                         , ""                             , ""                             , ""                           , ""                           , "",
    6  , "snake_Case"                , "snake_Case"                   , "snake_case"                   , "snakeCase"                  , "SnakeCase"                  , "SNAKE_CASE",
    7  , "_"                         , ""                             , ""                             , ""                           , ""                           , "",
    8  , "SNake"                     , "S_Nake"                       , "s_nake"                       , "sNake"                      , "SNake"                      , "S_NAKE",
    9  , "Snake"                     , "Snake"                        , "snake"                        , "snake"                      , "Snake"                      , "SNAKE",
    10 , "s_nake"                    , "s_nake"                       , "s_nake"                       , "sNake"                      , "SNake"                      , "S_NAKE",
    11 , "sn_ake"                    , "sn_ake"                       , "sn_ake"                       , "snAke"                      , "SnAke"                      , "SN_AKE",
    12 , "_"                         , ""                             , ""                             , ""                           , ""                           , "",
    13 , "SNaKE"                     , "S_Na_KE"                      , "s_na_ke"                      , "sNaKe"                      , "SNaKe"                      , "S_NA_KE",
    14 , "SNaKEr"                    , "S_Na_K_Er"                    , "s_na_k_er"                    , "sNaKEr"                     , "SNaKEr"                     , "S_NA_K_ER",
    15 , "s_na_k_er"                 , "s_na_k_er"                    , "s_na_k_er"                    , "sNaKEr"                     , "SNaKEr"                     , "S_NA_K_ER",
    16 , "_"                         , ""                             , ""                             , ""                           , ""                           , "",
    17 , "SNAKE SNAKE CASE"          , "SNAKE_SNAKE_CASE"             , "snake_snake_case"             , "snakeSnakeCase"             , "SnakeSnakeCase"             , "SNAKE_SNAKE_CASE",
    18 , "_"                         , ""                             , ""                             , ""                           , ""                           , "",
    19 , "snakeSnakECase"            , "snake_Snak_E_Case"            , "snake_snak_e_case"            , "snakeSnakECase"             , "SnakeSnakECase"             , "SNAKE_SNAK_E_CASE",
    20 , "SNAKE snakE_case"          , "SNAKE_snak_E_case"            , "snake_snak_e_case"            , "snakeSnakECase"             , "SnakeSnakECase"             , "SNAKE_SNAK_E_CASE",
    21 , "_"                         , ""                             , ""                             , ""                           , ""                           , "",
    22 , "ssRRss"                    , "ss_R_Rss"                     , "ss_r_rss"                     , "ssRRss"                     , "SsRRss"                     , "SS_R_RSS",
    23 , "ssRRRR"                    , "ss_RRRR"                      , "ss_rrrr"                      , "ssRrrr"                     , "SsRrrr"                     , "SS_RRRR",
    24 , "thisIsSomeCamelCase"       , "this_Is_Some_Camel_Case"      , "this_is_some_camel_case"      , "thisIsSomeCamelCase"        , "ThisIsSomeCamelCase"        , "THIS_IS_SOME_CAMEL_CASE",
    25 , "this.text"                 , "this_text"                    , "this_text"                    , "thisText"                   , "ThisText"                  , "THIS_TEXT",
    26 , "next.text"                 , "next_text"                    , "next_text"                    , "nextText"                   , "NextText"                  , "NEXT_TEXT",
    27 , "zip code"                  , "zip_code"                     , "zip_code"                     , "zipCode"                    , "ZipCode"                    , "ZIP_CODE",
    28 , "state"                     , "state"                        , "state"                        , "state"                      , "State"                      , "STATE",
    29 , "final count"               , "final_count"                  , "final_count"                  , "finalCount"                 , "FinalCount"                 , "FINAL_COUNT",
    30 , "BobDylanUSA"               , "Bob_Dylan_USA"                , "bob_dylan_usa"                , "bobDylanUsa"                , "BobDylanUsa"                , "BOB_DYLAN_USA",
    31 , "MikhailGorbachevUSSR"      , "Mikhail_Gorbachev_USSR"       , "mikhail_gorbachev_ussr"       , "mikhailGorbachevUssr"       , "MikhailGorbachevUssr"       , "MIKHAIL_GORBACHEV_USSR", 
    32 , "HelpfulStackOverflowPeople", "Helpful_Stack_Overflow_People", "helpful_stack_overflow_people", "helpfulStackOverflowPeople" , "HelpfulStackOverflowPeople" , "HELPFUL_STACK_OVERFLOW_PEOPLE", 
    33 , "IAmATallDrinkOfWater"      , "I_Am_A_Tall_Drink_Of_Water"   , "i_am_a_tall_drink_of_water"   , "iAmATallDrinkOfWater"       , "IAmATallDrinkOfWater"       , "I_AM_A_TALL_DRINK_OF_WATER",
    34 , "ICUDays"                   , "ICU_Days"                     , "icu_days"                     , "icuDays"                    , "IcuDays"                    , "ICU_DAYS",
    35 , "SexCode"                   , "Sex_Code"                     , "sex_code"                     , "sexCode"                    , "SexCode"                    , "SEX_CODE",
    36 , "MAX_of_MLD"                , "MAX_of_MLD"                   , "max_of_mld"                   , "maxOfMld"                   , "MaxOfMld"                   , "MAX_OF_MLD",
    37 , "Age.Group"                 , "Age_Group"                    , "age_group"                    , "ageGroup"                   , "AgeGroup"                  , "AGE_GROUP",
    38 , "ThisText"                  , "This_Text"                    , "this_text"                    , "thisText"                   , "ThisText"                   , "THIS_TEXT",
    39 , "NextText"                  , "Next_Text"                    , "next_text"                    , "nextText"                   , "NextText"                   , "NEXT_TEXT",
    40 , "test_123_ 1 1"             , "test_123_1_1"                 , "test_123_1_1"                 , "test123_1_1"                , "Test123_1_1"                , "TEST_123_1_1"
    )

# dat for arguments of to_any_case(). test non NULL arguments via dat. 
# test other three special case via specific examples
string <- c(NA, "_", "s_na_k_er", "SNAKE SNAKE CASE", "snakeSnakECase",
            "SNAKE snakE_case", "ssRRss", "ssRRRR", "thisIsSomeCamelCase",
            "this.text", "final count", "BobDylanUSA", "MikhailGorbachevUSSR",
            "HelpfulStackOverflowPeople", "ImATallDrinkOfWater", "ICUDays", "SexCode",
            "MAX_of_MLD", "Age.Group")
case <- c("parsed", "snake", "small_camel", "big_camel", "screaming_snake")
prefix <- c("", "start.")
postfix <- c("", ".end")
transliterations <- c("german", "Latin-ASCII")
  
dat <- expand.grid(string = string,
                     case = case,
                     postfix = postfix,
                     prefix = prefix,
                     transliterations = transliterations,
                     stringsAsFactors = FALSE)

# code to generate new results.
# purrrlyr::invoke_rows(snakecase::to_any_case, dat,
#                    preprocess = NULL,
#                    postprocess = NULL,
#                    .collate = "cols",
#                    .to = "output") %>% .$output %>% dput()

# Some Benchmarks:
# devtools::install_github("Tazinho/snakecase", force = TRUE)
# library(snakecase)
# 
# string_gen <- function(times){paste0("str", 1:times)}
# other_gen <- function(times){paste0("other", 1:times)}
# 
# str10 <- string_gen(10)
# str1000 <- string_gen(1000)
# oth10 <- other_gen(10)
# oth1000 <- other_gen(1000)
# 
# microbenchmark::microbenchmark(
#   to_any_case(string = str10, case = "snake", preprocess = oth10),
#   to_any_case(string = str1000, case = "snake", preprocess = oth1000)
# )
# 
# microbenchmark::microbenchmark(
#   to_any_case(string = str10, case = "snake", postprocess = oth10),
#   to_any_case(string = str1000, case = "snake", postprocess = oth1000)
# )
# 
# microbenchmark::microbenchmark(
#   to_any_case(string = str10, case = "snake",   prefix = oth10),
#   to_any_case(string = str1000, case = "snake", prefix = oth1000)
# )
# 
# microbenchmark::microbenchmark(
#   to_any_case(string = str10, case = "snake"  , postfix = oth10),
#   to_any_case(string = str1000, case = "snake", postfix = oth1000)
# )
# 
# microbenchmark::microbenchmark(
#   to_any_case(string = str10, case = "snake"  , protect = oth10),
#   to_any_case(string = str1000, case = "snake", protect = oth1000)
# )

#' Internal helper to test the design rules for any string and setting of \code{to_any_case()}
#'
#' @param string A string (for example names of a data frame).
#' @param sep_in String that will be wrapped internally into \code{stringr::regex()}. 
#' All matches will be treated as additional splitting parameters besides the default ones 
#' (\code{"_"} and \code{" "}), when parsing the input string.
#' @param transliterations A character vector (if not \code{NULL}). The entries of this argument
#' need to be elements of \code{stringi::stri_trans_list()} (like "Latin-ASCII", which is often useful) or names of lookup tables (currently
#' only "german" is supported). In the order of the entries the letters of the input
#'  string will be transliterated via \code{stringi::stri_trans_general()} or replaced via the 
#'  matches of the lookup table.
#' @param sep_out String that will be used as separator. The defaults are \code{"_"} 
#' and \code{""}, regarding the specified \code{case}.
#' @param prefix prefix (string).
#' @param postfix postfix (string).
#' @param empty_fill A string. If it is supplied, then each entry that matches "" will be replaced
#' by the supplied string to this argument.
#' @param unique_sep A string. If it is supplied, then duplicated names will get a suffix integer
#' in the order of their appearance. The suffix is separated by the supplied string to this argument.
#' @param parsing_option An integer that will determine the parsing_option.
#' \itemize{
#'  \item{1: \code{RRRStudio -> RRR_Studio}}
#'  \item{2: \code{RRRStudio -> RRRS_tudio}}
#'  \item{3: parses at the beginning like option 1 and the rest like option 2.}
#'  \item{4: parses at the beginning like option 2 and the rest like option 1.}
#'  \item{5: parses like option 1 but suppresses "_" around non special characters.
#'  In this way case conversion won't apply after these characters. See examples.}
#'  \item{6: parses like option 1, but digits directly behind/in front non-digits, will stay as is.}
#'  \item{any other integer <= 0: no parsing"}
#'  }
#' 
#' @return A character vector separated by underscores, containing the parsed string.
#'
#' @author Malte Grosser, \email{malte.grosser@@gmail.com}
#' @keywords utilities
#'
check_design_rule <- function(string, sep_in = NULL,
                             transliterations = NULL, sep_out = NULL, prefix = "",
                             postfix = "", unique_sep = NULL, empty_fill = NULL, parsing_option = 1){
  test_c <- function(string, case){
    to_any_case(string = string, case = case, sep_in = sep_in, 
                transliterations = transliterations,
                sep_out = sep_out, prefix = prefix, postfix = postfix,
                unique_sep = unique_sep, empty_fill = empty_fill, 
                parsing_option = parsing_option)
  }
  all(
    # snake
    test_c(string, case = "snake") == test_c(test_c(string, case = "snake"), case = "snake"),
    test_c(string, case = "snake") == test_c(test_c(string, case = "small_camel"), case = "snake"),
    test_c(string, case = "snake") == test_c(test_c(string, case = "big_camel"), case = "snake"),
    test_c(string, case = "snake") == test_c(test_c(string, case = "screaming_snake"), case = "snake"),
    # small_camel
    test_c(string, case = "small_camel") == test_c(test_c(string, case = "snake"), case = "small_camel"),
    test_c(string, case = "small_camel") == test_c(test_c(string, case = "small_camel"), case = "small_camel"),
    test_c(string, case = "small_camel") == test_c(test_c(string, case = "big_camel"), case = "small_camel"),
    test_c(string, case = "small_camel") == test_c(test_c(string, case = "screaming_snake"), case = "small_camel"),
    # big_camel
    test_c(string, case = "big_camel") == test_c(test_c(string, case = "snake"), case = "big_camel"),
    test_c(string, case = "big_camel") == test_c(test_c(string, case = "small_camel"), case = "big_camel"),
    test_c(string, case = "big_camel") == test_c(test_c(string, case = "big_camel"), case = "big_camel"),
    test_c(string, case = "big_camel") == test_c(test_c(string, case = "screaming_snake"), case = "big_camel"),
    # screaming_snake
    test_c(string, case = "screaming_snake") == test_c(test_c(string, case = "snake"), case = "screaming_snake"),
    test_c(string, case = "screaming_snake") == test_c(test_c(string, case = "small_camel"), case = "screaming_snake"),
    test_c(string, case = "screaming_snake") == test_c(test_c(string, case = "big_camel"), case = "screaming_snake"),
    test_c(string, case = "screaming_snake") == test_c(test_c(string, case = "screaming_snake"), case = "screaming_snake")
  )
  }

#' Specific case converter shortcuts
#' 
#' Wrappers around \code{to_any_case()}
#'
#' @param string A string (for example names of a data frame).
#'  
#' @param abbreviations character. (Case insensitive) matched abbreviations are surrounded by underscores. In this way, they can get recognized by the parser. This is useful when e.g. \code{parsing_option} 1 is needed for the use case, but some abbreviations but some substrings would require \code{parsing_option} 2. Furthermore, this argument also specifies the formatting of abbreviations in the output for the cases title, mixed, lower and upper camel. E.g. for upper camel the first letter is always in upper case, but when the abbreviation is supplied in upper case, this will also be visible in the output.
#'  
#'  Use this feature with care: One letter abbreviations and abbreviations next to each other are hard to read and also not easy to parse for further processing.
#'  
#' @param sep_in (short for separator input) if character, is interpreted as a
#'  regular expression (wrapped internally into \code{stringr::regex()}). 
#'  The default value is a regular expression that matches any sequence of
#'  non-alphanumeric values. All matches will be replaced by underscores 
#'  (additionally to \code{"_"} and \code{" "}, for which this is always true, even
#'  if \code{NULL} is supplied). These underscores are used internally to split
#'  the strings into substrings and specify the word boundaries.
#' 
#' @param parsing_option An integer that will determine the parsing_option.
#' \itemize{
#'  \item{1: \code{"RRRStudio" -> "RRR_Studio"}}
#'  \item{2: \code{"RRRStudio" -> "RRRS_tudio"}}
#'  \item{3: \code{"RRRStudio" -> "RRRSStudio"}. This will become for example \code{"Rrrstudio"} when we convert to lower camel case.}
#'  \item{-1, -2, -3: These \code{parsing_options}'s will suppress the conversion after non-alphanumeric values.}
#'  \item{0: no parsing}
#'  }
#' 
#' @param transliterations A character vector (if not \code{NULL}). The entries of this argument
#' need to be elements of \code{stringi::stri_trans_list()} (like "Latin-ASCII", which is often useful) or names of lookup tables (currently
#' only "german" is supported). In the order of the entries the letters of the input
#'  string will be transliterated via \code{stringi::stri_trans_general()} or replaced via the 
#'  matches of the lookup table. When named character elements are supplied as part of `transliterations`, anything that matches the names is replaced by the corresponding value.

#' You should use this feature with care in case of \code{case = "parsed"}, \code{case = "internal_parsing"} and 
#' \code{case = "none"}, since for upper case letters, which have transliterations/replacements
#'  of length 2, the second letter will be transliterated to lowercase, for example Oe, Ae, Ss, which
#'  might not always be what is intended. In this case you can make usage of the option to supply named elements and specify the transliterations yourself.
#' 
#' @param numerals A character specifying the alignment of numerals (\code{"middle"}, \code{left}, \code{right} or \code{asis}). I.e. \code{numerals = "left"} ensures that no output separator is in front of a digit.
#' 
#' @param sep_out (short for separator output) String that will be used as separator. The defaults are \code{"_"} 
#' and \code{""}, regarding the specified \code{case}. When \code{length(sep_out) > 1}, the last element of \code{sep_out} gets recycled and separators are incorporated per string according to their order.
#' 
#' @param unique_sep A string. If not \code{NULL}, then duplicated names will get 
#' a suffix integer
#' in the order of their appearance. The suffix is separated by the supplied string
#'  to this argument.
#'  
#' @param empty_fill A string. If it is supplied, then each entry that matches "" will be replaced
#' by the supplied string to this argument.
#'  
#' @param prefix prefix (string).
#' 
#' @param postfix postfix (string).
#' 
#' @return A character vector according the specified parameters above.
#'
#' @note caseconverters are vectorised over \code{string}, \code{sep_in}, \code{sep_out},
#'  \code{empty_fill}, \code{prefix} and \code{postfix}.
#' @author Malte Grosser, \email{malte.grosser@@gmail.com}
#' @keywords utilities
#' 
#' @name caseconverter
#' @return A character vector according the specified target case.
#' 
#' @author Malte Grosser, \email{malte.grosser@@gmail.com}
#' @keywords utilities
#' 
#' @examples
#' strings <- c("this Is a Strange_string", "AND THIS ANOTHER_One", NA)
#' 
#' to_snake_case(strings)
#' to_lower_camel_case(strings)
#' to_upper_camel_case(strings)
#' to_screaming_snake_case(strings)
#' to_lower_upper_case(strings)
#' to_upper_lower_case(strings)
#' to_parsed_case(strings)
#' to_mixed_case(strings)
#' to_swap_case(strings)
#' to_sentence_case(strings)
#' to_random_case(strings)
#' to_title_case(strings)
#' 
#' 
#' @rdname caseconverter
#' @seealso \href{https://github.com/Tazinho/snakecase}{snakecase on github}, \code{\link{to_any_case}} for flexible high level conversion and more examples.
#' @export

to_snake_case <- function(string,
                          abbreviations = NULL,
                          sep_in = "[^[:alnum:]]",
                          parsing_option = 1,
                          transliterations = NULL,
                          numerals = "middle",
                          sep_out = NULL,
                          unique_sep = NULL,
                          empty_fill = NULL,
                          prefix = "",
                          postfix = ""){
  to_any_case(string = string,
              case = "snake",
              sep_in = sep_in,
              transliterations = transliterations,     
              numerals = numerals,
              sep_out = sep_out,
              prefix = prefix,
              postfix = postfix,
              unique_sep = unique_sep,
              empty_fill = empty_fill,
              parsing_option = parsing_option, 
              abbreviations = abbreviations)
}

#' @rdname caseconverter
#' @export

to_lower_camel_case <- function(string,
                                abbreviations = NULL,
                                sep_in = "[^[:alnum:]]",
                                parsing_option = 1,
                                transliterations = NULL,   
                                numerals = "middle", 
                                sep_out = NULL,
                                unique_sep = NULL,
                                empty_fill = NULL,
                                prefix = "",
                                postfix = ""){
  to_any_case(string = string,
              case = "lower_camel",
              sep_in = sep_in,
              transliterations = transliterations,               
              numerals = numerals,
              sep_out = sep_out,
              prefix = prefix,
              postfix = postfix,
              unique_sep = unique_sep,
              empty_fill = empty_fill,
              parsing_option = parsing_option, 
              abbreviations = abbreviations)
}

#' @rdname caseconverter
#' @export

to_upper_camel_case <- function(string,
                                abbreviations = NULL,
                                sep_in = "[^[:alnum:]]",
                                parsing_option = 1,
                                transliterations = NULL,                           
                                numerals = "middle",                                  
                                sep_out = NULL,
                                unique_sep = NULL,
                                empty_fill = NULL,
                                prefix = "",
                                postfix = ""){
  to_any_case(string = string,
              case = "upper_camel",
              sep_in = sep_in,
              transliterations = transliterations,               
              numerals = numerals,
              sep_out = sep_out,
              prefix = prefix,
              postfix = postfix,
              unique_sep = unique_sep,
              empty_fill = empty_fill,
              parsing_option = parsing_option, 
              abbreviations = abbreviations)
}

#' @rdname caseconverter
#' @export 

to_screaming_snake_case <- function(string,
                                    abbreviations = NULL,
                                    sep_in = "[^[:alnum:]]",
                                    parsing_option = 1,
                                    transliterations = NULL,                      
                                    numerals = "middle",                                  
                                    sep_out = NULL,
                                    unique_sep = NULL,
                                    empty_fill = NULL,
                                    prefix = "",
                                    postfix = ""){
  to_any_case(string = string,
              case = "screaming_snake",
              sep_in = sep_in,
              transliterations = transliterations,               
              numerals = numerals,
              sep_out = sep_out,
              prefix = prefix,
              postfix = postfix,
              unique_sep = unique_sep,
              empty_fill = empty_fill,
              parsing_option = parsing_option, 
              abbreviations = abbreviations)
}

#' @rdname caseconverter
#' @export

to_parsed_case <- function(string,
                           abbreviations = NULL,
                           sep_in = "[^[:alnum:]]",
                           parsing_option = 1,
                           transliterations = NULL,                     
                           numerals = "middle",                                  
                           sep_out = NULL,
                           unique_sep = NULL,
                           empty_fill = NULL,
                           prefix = "",
                           postfix = ""){
  to_any_case(string = string,
              case = "parsed",
              sep_in = sep_in,
              transliterations = transliterations,               
              numerals = numerals,
              sep_out = sep_out,
              prefix = prefix,
              postfix = postfix,
              unique_sep = unique_sep,
              empty_fill = empty_fill,
              parsing_option = parsing_option,
              abbreviations = abbreviations)
}

#' @rdname caseconverter
#' @export

to_mixed_case <- function(string,
                          abbreviations = NULL,
                          sep_in = "[^[:alnum:]]",
                          parsing_option = 1,
                          transliterations = NULL,               
                          numerals = "middle",   
                          sep_out = NULL,
                          unique_sep = NULL,
                          empty_fill = NULL,
                          prefix = "",
                          postfix = ""){
  to_any_case(string = string,
              case = "mixed",
              sep_in = sep_in,
              transliterations = transliterations,               
              numerals = numerals,
              sep_out = sep_out,
              prefix = prefix,
              postfix = postfix,
              unique_sep = unique_sep,
              empty_fill = empty_fill,
              parsing_option = parsing_option,
              abbreviations = abbreviations)
}

#' @rdname caseconverter
#' @export

to_lower_upper_case <- function(string,
                                abbreviations = NULL,
                                sep_in = "[^[:alnum:]]",
                                parsing_option = 1,
                                transliterations = NULL,        
                                numerals = "middle",   
                                sep_out = NULL,
                                unique_sep = NULL,
                                empty_fill = NULL,
                                prefix = "",
                                postfix = ""){
  to_any_case(string = string,
              case = "lower_upper",
              sep_in = sep_in,
              transliterations = transliterations,  
              numerals = numerals,
              sep_out = sep_out,
              prefix = prefix,
              postfix = postfix,
              unique_sep = unique_sep,
              empty_fill = empty_fill,
              parsing_option = parsing_option,
              abbreviations = abbreviations)
}

#' @rdname caseconverter
#' @export

to_upper_lower_case <- function(string,
                                abbreviations = NULL,
                                sep_in = "[^[:alnum:]]",
                                parsing_option = 1,
                                transliterations = NULL,  
                                numerals = "middle",  
                                sep_out = NULL,
                                unique_sep = NULL,
                                empty_fill = NULL,
                                prefix = "",
                                postfix = ""){
  to_any_case(string = string,
              case = "upper_lower",
              sep_in = sep_in,
              transliterations = transliterations,            
              numerals = numerals,
              sep_out = sep_out,
              prefix = prefix,
              postfix = postfix,
              unique_sep = unique_sep,
              empty_fill = empty_fill,
              parsing_option = parsing_option,
              abbreviations = abbreviations)
}

#' @rdname caseconverter
#' @export

to_swap_case <- function(string,
                         abbreviations = NULL,
                         sep_in = "[^[:alnum:]]",
                         parsing_option = 1,
                         transliterations = NULL,             
                         numerals = "middle",                   
                         sep_out = NULL,
                         unique_sep = NULL,
                         empty_fill = NULL,
                         prefix = "",
                         postfix = ""){
  to_any_case(string = string,
              case = "swap",
              sep_in = sep_in,
              transliterations = transliterations,              
              numerals = numerals,
              sep_out = sep_out,
              prefix = prefix,
              postfix = postfix,
              unique_sep = unique_sep,
              empty_fill = empty_fill,
              parsing_option = parsing_option,
              abbreviations = abbreviations)
}

#' @rdname caseconverter
#' @export

to_sentence_case <- function(string,
                         abbreviations = NULL,
                         sep_in = "[^[:alnum:]]",
                         parsing_option = 1,
                         transliterations = NULL,       
                         numerals = "middle",                     
                         sep_out = NULL,
                         unique_sep = NULL,
                         empty_fill = NULL,
                         prefix = "",
                         postfix = ""){
  to_any_case(string = string,
              case = "sentence",
              sep_in = sep_in,
              transliterations = transliterations,  
              numerals = numerals,
              sep_out = sep_out,
              prefix = prefix,
              postfix = postfix,
              unique_sep = unique_sep,
              empty_fill = empty_fill,
              parsing_option = parsing_option,
              abbreviations = abbreviations)
}

#' @rdname caseconverter
#' @export

to_random_case <- function(string,
                         abbreviations = NULL,
                         sep_in = "[^[:alnum:]]",
                         parsing_option = 1,
                         transliterations = NULL,             
                         numerals = "middle",                   
                         sep_out = NULL,
                         unique_sep = NULL,
                         empty_fill = NULL,
                         prefix = "",
                         postfix = ""){
  to_any_case(string = string,
              case = "random",
              sep_in = sep_in,
              transliterations = transliterations,              
              numerals = numerals,
              sep_out = sep_out,
              prefix = prefix,
              postfix = postfix,
              unique_sep = unique_sep,
              empty_fill = empty_fill,
              parsing_option = parsing_option,
              abbreviations = abbreviations)
}

#' @rdname caseconverter
#' @export

to_title_case <- function(string,
                         abbreviations = NULL,
                         sep_in = "[^[:alnum:]]",
                         parsing_option = 1,
                         transliterations = NULL,             
                         numerals = "middle",                   
                         sep_out = NULL,
                         unique_sep = NULL,
                         empty_fill = NULL,
                         prefix = "",
                         postfix = ""){
  to_any_case(string = string,
              case = "title",
              sep_in = sep_in,
              transliterations = transliterations,              
              numerals = numerals,
              sep_out = sep_out,
              prefix = prefix,
              postfix = postfix,
              unique_sep = unique_sep,
              empty_fill = empty_fill,
              parsing_option = parsing_option,
              abbreviations = abbreviations)
}

context("to_sentence_case")

test_that("basic_example",{
  
  expect_equal(
    to_sentence_case("bla bla_bal"),
    "Bla bla bal")
  
  expect_equal(to_sentence_case("the_boy_likes_snake_case",
                                transliterations = c("boy" = "baby",
                                                     "snake" = "screaming__snake"),
                                sep_out = " "),
               "The baby likes screaming snake case")
  
  expect_equal(to_sentence_case("the_boy_likes_snake_case",
                       transliterations = c("boy" = "baby",
                                            "snake" = "screaming__snake")),
    "The baby likes screaming snake case")
})

context("to_upper_lower_case")

test_that("random stuff", {
  expect_equal(to_lower_upper_case("RStudioRRRStudio"),
               "rSTUDIOrrrSTUDIO")
  }
)

test_that("preserve-name-attribute",{
  labs <- c(a = "abcDEF", b = "bbccEE", c = "TeESt it")
  
  expect_equal(
    to_lower_upper_case(labs),
    structure(c("abcDEF", "bbccEE", "teEstIT"), .Names = c("a", "b", 
                                                           "c"))
  )
})

context("to_mixed_case")

test_that("random stuff", {
  expect_equal(to_mixed_case("RStudioRRRStudio"),
               "R_Studio_Rrr_Studio")
  }
)

test_that("preserve-name-attribute",{
  labs <- c(a = "abcDEF", b = "bbccEE", c = "TeESt it")
  
  expect_equal(
    to_mixed_case(labs),
    structure(c("abc_Def", "bbcc_Ee", "Te_E_St_it"), .Names = c("a", 
                                                                "b", "c"))
  )
})

context("to_lower_camel_case")

test_that("examples", {
  expect_equal(to_lower_camel_case(cases[["examples"]]),
               cases[["small_camel_case"]])}
)

test_that("rules",{
  examples <- cases[["examples"]]
  
  expect_equal(to_lower_camel_case(to_snake_case(examples)),
               to_lower_camel_case(examples)
  ) 
  expect_equal(to_lower_camel_case(to_lower_camel_case(examples)),
               to_lower_camel_case(examples)
  ) 
  expect_equal(to_lower_camel_case(to_upper_camel_case(examples)),
               to_lower_camel_case(examples)
  ) 
  expect_equal(to_lower_camel_case(to_screaming_snake_case(examples)),
               to_lower_camel_case(examples)
  ) 
  expect_equal(to_lower_camel_case(to_parsed_case(examples)),
               to_lower_camel_case(examples)
               )
})

test_that("preserve-name-attribute",{
  labs <- c(a = "abcDEF", b = "bbccEE", c = "TeESt it")
  
  expect_equal(
    to_lower_camel_case(labs),
    structure(c("abcDef", "bbccEe", "teEStIt"), .Names = c("a", "b", 
                                                           "c"))
  )
})

test_that("abbreviations", {
  expect_equal(to_lower_camel_case("t2d_status", abbreviations = "t2d"),
               "t2dStatus")
})

context("to_upper_camel_case")

test_that("examples", {
  expect_equal(to_upper_camel_case(cases[["examples"]]),
               cases[["big_camel_case"]])}
)

test_that("rules",{
  examples <- cases[["examples"]]
  
  expect_equal(to_upper_camel_case(to_snake_case(examples)),
               to_upper_camel_case(examples)
  ) 
  expect_equal(to_upper_camel_case(to_lower_camel_case(examples)),
               to_upper_camel_case(examples)
  ) 
  expect_equal(to_upper_camel_case(to_upper_camel_case(examples)),
               to_upper_camel_case(examples)
  ) 
  expect_equal(to_upper_camel_case(to_screaming_snake_case(examples)),
               to_upper_camel_case(examples)
  ) 
  expect_equal(to_upper_camel_case(to_parsed_case(examples)),
               to_upper_camel_case(examples)
  ) 
})

test_that("preserve-names-attribute",{
  labs <- c(a = "abcDEF", b = "bbccEE", c = "TeESt it")
  
  expect_equal(to_upper_camel_case(labs),
              structure(c("AbcDef", "BbccEe", "TeEStIt"), 
                        .Names = c("a", "b", "c")))
})

test_that("parsing_options",{
  expect_equal(to_upper_camel_case("look_AfterThe-hyphen andThe.dot", parsing_option = 1, numerals = "asis", sep_in = NULL),
               "LookAfterThe-HyphenAndThe.Dot")
  
  expect_equal(to_upper_camel_case("look_AfterThe-hyphen andThe.dot", parsing_option = -1, numerals = "asis", sep_in = NULL),
               "LookAfterThe-hyphenAndThe.dot")
  
  expect_equal(to_upper_camel_case("look_AfterThe-hyphen andThe.dot", parsing_option = 2, numerals = "asis", sep_in = NULL),
               "LookAfterThe-HyphenAndThe.Dot")
  
  expect_equal(to_upper_camel_case("look_AfterThe-hyphen andThe.dot", parsing_option = -2, numerals = "asis", sep_in = NULL),
               "LookAfterThe-hyphenAndThe.dot")
  
  expect_equal(to_upper_camel_case("look_AfterThe-hyphen andThe.dot", parsing_option = 3, numerals = "asis", sep_in = NULL),
               "LookAfterThe-HyphenAndThe.Dot")
  
  expect_equal(to_upper_camel_case("look_AfterThe-hyphen andThe.dot", parsing_option = -3, numerals = "asis", sep_in = NULL),
               "LookAfterThe-hyphenAndThe.dot")
})

context("to_snake_case")

test_that("examples", {
  expect_equal(to_snake_case(cases[["examples"]]),
               cases[["snake_case"]])}
  )

test_that("rules",{
  examples <- cases[["examples"]]
  
  expect_equal(to_snake_case(to_snake_case(examples)),
               to_snake_case(examples)
               ) 
  expect_equal(to_snake_case(to_lower_camel_case(examples)),
               to_snake_case(examples)
  ) 
  expect_equal(to_snake_case(to_upper_camel_case(examples)),
               to_snake_case(examples)
  ) 
  expect_equal(to_snake_case(to_screaming_snake_case(examples)),
               to_snake_case(examples)
  ) 
  expect_equal(to_snake_case(to_parsed_case(examples)),
               to_snake_case(examples)
  ) 
})

test_that("preserve-name-attribute",{
  labs <- c(a = "abcDEF", b = "bbccEE", c = "TeESt it")
  
  expect_equal(
    to_snake_case(labs),
    structure(c("abc_def", "bbcc_ee", "te_e_st_it"), .Names = c("a", 
                                                                "b", "c"))
  )
})

context("to_any_case")

test_that("examples", {
  examples <- cases[["examples"]]
  
  expect_equal(to_any_case(examples, case = "snake"),
               cases[["snake_case"]])
  
  expect_equal(to_any_case(examples, case = "small_camel"),
               cases[["small_camel_case"]])
  
  expect_equal(to_any_case(examples, case = "big_camel"),
               cases[["big_camel_case"]])
  
  expect_equal(to_any_case(examples, case = "screaming_snake"),
               cases[["screaming_snake_case"]])
  
  expect_equal(to_any_case(examples, case = "parsed"),
             cases[["parsed_case"]])
  
  expect_equal(to_any_case("R.Studio", case = "big_camel", sep_out = "-"),
               "R-Studio")
  
  expect_equal(to_any_case("HAMBURGcity", case = "parsed", parsing_option = 0),
               "HAMBURGcity")
  
  expect_equal(to_any_case(c("RSSfeedRSSfeed", "USPassport", "USpassport"), abbreviations = c("RSS", "US")),
               c("rss_feed_rss_feed", "us_passport", "us_passport"))
  
  expect_equal(to_any_case("NBAGame", abbreviations = "NBA", case = "parsed"),
               "NBA_Game")
  expect_equal(to_any_case("NBAGame", abbreviations = "NBA", case = "mixed"),
               "NBA_Game")
  expect_equal(to_any_case("NBAGame", abbreviations = "NBA", case = "snake"),
               "nba_game")
  expect_equal(to_any_case("NBAGame", abbreviations = "NBA", case = "screaming_snake"),
               "NBA_GAME")
  expect_equal(to_any_case("NBAGame", abbreviations = "NBA", case = "internal_parsing"),
               "NBA_Game")
  expect_equal(to_any_case("NBAGame", abbreviations = "NBA", case = "upper_camel"),
               "NBAGame")
  expect_equal(to_any_case("nba_game", abbreviations = "NBA", case = "lower_camel"),
               "nbaGame")
  expect_equal(to_any_case("NBA_game", abbreviations = "NBA", case = "upper_camel"),
               "NBAGame")
  expect_equal(to_any_case("nba_game", abbreviations = "NBA", case = "upper_camel"),
               "NBAGame")
  expect_equal(to_any_case("NBA_game_NBA", abbreviations = "NBA", case = "lower_camel"),
               "nbaGameNBA")
  expect_equal(to_any_case("NBA_game_NBA", abbreviations = "NBA", case = "lower_upper"),
               "nbaGAMEnba")
  expect_equal(to_any_case("NBA_game", abbreviations = "NBA", case = "mixed"),
               "NBA_game")
  expect_equal(to_any_case("NBA_game_NBA", abbreviations = "NBA", case = "mixed"),
               "NBA_game_NBA")
  expect_equal(to_snake_case(c("NBAGame", "NBAgame"), abbreviations = "NBA"),
               c("nba_game", "nba_game"))
  expect_equal(to_upper_camel_case(c("nba_game", "nba_game"), abbreviations = "NBA"),
               c("NBAGame", "NBAGame"))
  }
)

test_that("numerals_tight",
          {
            expect_equal(to_any_case("bla_la_123_123_1_bla", numerals = "tight"),
                         "bla_la123_123_1bla")
          })

test_that("attributes", {
  expect_equal(
    {strings <- c("this Is a Strange_string", "AND THIS ANOTHER_One");
    names(strings) <- c("String A", "String B");
    attr(strings, "test.attr") <- "test";
    strings},
    structure(c(`String A` = "this Is a Strange_string", `String B` = "AND THIS ANOTHER_One"),
              test.attr = "test"))
    }
  )

test_that("numerals", {
  expect_equal(to_any_case("123bla123bla_434bla"),
               "123_bla_123_bla_434_bla")
  
  expect_equal(to_any_case("123bla123bla_434bla", 
                           numerals = "asis"),
               "123bla123bla_434bla")
  
  expect_equal(to_any_case("123bla123bla_434bla",
                           numerals = "left"),
               "123_bla123_bla434_bla")

  expect_equal(to_any_case("123bla123bla_434bla", 
                           numerals = "right"),
               "123bla_123bla_434bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla", numerals = "middle"),
               "123_bla_123_123_bla_434_bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla", numerals = "asis"),
               "123bla123_123bla_434bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla", numerals = "left"),
               "123_bla123_123_bla434_bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla",
                           numerals = "right"),
               "123bla_123_123bla_434bla")

  expect_equal(to_any_case("123bla123_123bla_434bla", numerals = "right"), 
               "123bla_123_123bla_434bla")

  expect_equal(to_any_case("123bla123_123bla_434bla", numerals = "right"),
               "123bla_123_123bla_434bla")
  
  expect_equal(to_upper_camel_case("123bla123_123bla_434bla"),
               "123Bla123_123Bla434Bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla", case = "upper_camel", numerals = "middle"),
    "123Bla123_123Bla434Bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla", case = "upper_camel", numerals = "asis"), "123Bla123_123Bla434Bla")

  expect_equal(to_any_case("123bla123_123bla_434bla", case = "upper_camel", numerals = "left"), 
               "123Bla123_123Bla434Bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla", case = "upper_camel", numerals = "right"),
               "123Bla123_123Bla434Bla")

  expect_equal(to_any_case("123bla123_123bla_434bla", case = "parsed", numerals = "middle"),
               "123_bla_123_123_bla_434_bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla", case = "parsed", numerals = "asis"),
               "123bla123_123bla_434bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla", case = "parsed", numerals = "left"),
               "123_bla123_123_bla434_bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla",
                           case = "parsed",
                           numerals = "right"),
    "123bla_123_123bla_434bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla", 
                           case = "none",
                           numerals = "middle"),
               "123bla123_123bla_434bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla", 
                           case = "none", 
                           numerals = "asis"),
               "123bla123_123bla_434bla")

  expect_equal(to_any_case("123bla123_123bla_434bla",
                           case = "none",
                           numerals = "left"),
               "123bla123_123bla_434bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla", 
                           case = "none",
                           numerals = "right"),
               "123bla123_123bla_434bla")

  expect_equal(to_any_case("123bla123_123bla_434bla", 
                           numerals = "middle"),
               "123_bla_123_123_bla_434_bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla", numerals = "asis"),
               "123bla123_123bla_434bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla",
                           numerals = "left"),
               "123_bla123_123_bla434_bla")
  
  expect_equal(to_any_case("123bla123_123bla_434bla",
                           numerals = "right"),
               "123bla_123_123bla_434bla")
  
  expect_equal(to_any_case("species42value 23month",
                           numerals = "asis"),
               "species42value_23month")
  
  expect_equal(to_any_case(c("HHcity", "IDtable1", "KEYtable2", "newUSelections"),
                           parsing_option = 2,
                           numerals = "middle"),
               c("hh_city", "id_table_1", "key_table_2", "new_us_elections"))
  
  expect_equal(to_any_case(c("HHcity", "IDtable1", "KEYtable2", "newUSelections"),
                           parsing_option = 2, 
                           numerals = "asis"),
               c("hh_city","id_table1", "key_table2", "new_us_elections"))
  
  expect_equal(to_any_case(c("HHcity", "IDtable1", "KEYtable2", "newUSelections"), parsing_option = 2, numerals = "left"),
               c("hh_city", "id_table1", "key_table2","new_us_elections"))

  expect_equal(to_any_case(c("HHcity", "IDtable1", "KEYtable2", "newUSelections"),
                           parsing_option = 2,
                           numerals = "right"),
               c("hh_city", "id_table_1", "key_table_2",
                 "new_us_elections"))
})

test_that("preserve-names-attribute", {
  
  labs <- c(a = "abcDEF", b = "bbccEE", c = "TeESt it")
  
  expect_equal(
    to_any_case(labs, case = "snake"),
    structure(c("abc_def", "bbcc_ee", "te_e_st_it"), .Names = c("a", "b", "c"))
    )
  
  expect_equal(
    to_any_case(labs, case = "upper_camel"),
    structure(c("AbcDef", "BbccEe", "TeEStIt"), .Names = c("a", "b", "c"))
    )
  
  expect_equal(
    to_any_case(labs, case = "lower_camel"),
    structure(c("abcDef", "bbccEe", "teEStIt"), .Names = c("a", "b", "c"))
    )
  
  expect_equal(
    to_any_case(labs, case = "parsed"),
    structure(c("abc_DEF", "bbcc_EE", "Te_E_St_it"), .Names = c("a", "b", "c"))
  )
  
  expect_equal(
    to_any_case(labs, case = "mixed"),
    structure(c("abc_Def", "bbcc_Ee", "Te_E_St_it"), .Names = c("a", "b", "c"))
  )
  
  expect_equal(
    to_any_case(labs, case = "none", sep_in = NULL), 
    structure(c("abcDEF", "bbccEE", "TeESt it"), .Names = c("a", "b", "c"))
    )
  
  expect_equal(
    to_any_case(labs, case = "all_caps"),
    structure(c("ABC_DEF", "BBCC_EE", "TE_E_ST_IT"), .Names = c("a", "b", "c"))
    )
  
  expect_equal(
    to_any_case(labs, case = "upper_lower"),
    structure(c("ABCdef", "BBCCee", "TEeSTit"), .Names = c("a", "b", "c"))
    )
  
  expect_equal(
    to_any_case(labs, case = "lower_upper"),
    structure(c("abcDEF", "bbccEE", "teEstIT"), .Names = c("a", "b", "c"))
    )
})

test_that("uniqe_sep", {
  expect_equal(
    to_any_case(c("bla", "bla"), unique_sep = "_"),
    c("bla",   "bla_1")
  )
  
  expect_equal(
    to_any_case(c("bla", "bla"), unique_sep = NULL),
    c("bla", "bla")
  )
})

test_that("sentence_case", {
  expect_equal(
  to_sentence_case("this_is_a_sentence", sep_out = " ", postfix = "."),
  "This is a sentence.")
})

test_that("janitor-pkg-tests",{
  skip_if_not( l10n_info()$`UTF-8`)
  
  clean_names3 <- function(old_names, case = "snake"){
    new_names <- gsub("'", "", old_names) # remove quotation marks
    new_names <- gsub("\"", "", new_names) # remove quotation marks
    new_names <- gsub("%", ".percent_", new_names)
    new_names <- gsub("#", ".number_", new_names)
    new_names <- gsub("^[[:space:][:punct:]]+", "", new_names)
    new_names <- make.names(new_names)
    new_names <- to_any_case(new_names, case = case, sep_in = "\\.", 
                  transliterations = c("Latin-ASCII"), parsing_option = 1, numerals = "asis")
    # Handle duplicated names - they mess up dplyr pipelines
    # This appends the column number to repeated instances of duplicate variable names
    dupe_count <- vapply(1:length(new_names), function(i) { 
      sum(new_names[i] == new_names[1:i]) }, integer(1))
    
    new_names[dupe_count > 1] <- paste(new_names[dupe_count > 1],
                                       dupe_count[dupe_count > 1],
                                       sep = "_")
    new_names
  }
  
  expect_equal(clean_names3(c("sp ace", "repeated", "a**^@", "%", "*",
                 "!", "d(!)9", "REPEATED", "can\"'t", "hi_`there`",
                 "  leading spaces", "\u20AC", "a\u00E7\u00E3o", "far\u0153", "r.st\u00FCdio:v.1.0.143")),
               c("sp_ace", "repeated", "a", "percent", "x", "x_2", "d_9", "repeated_2", 
                 "cant", "hi_there", "leading_spaces", "x_3", "acao", "faroe", 
                 "r_studio_v_1_0_143"))
  
  expect_equal(clean_names3(c("sp ace", "repeated", "a**^@", "%", "*",
                              "!", "d(!)9", "REPEATED", "can\"'t", "hi_`there`",
                              "  leading spaces", "\u20AC", "a\u00E7\u00E3o", "far\u0153", "r.st\u00FCdio:v.1.0.143"),
                            case = "parsed"),
               c("sp_ace", "repeated", "a", "percent", "X", "X_2", "d_9", "REPEATED", 
                 "cant", "hi_there", "leading_spaces", "X_3", "acao", "faroe", 
                 "r_studio_v_1_0_143"))
  
  expect_equal(clean_names3(c("sp ace", "repeated", "a**^@", "%", "*",
                              "!", "d(!)9", "REPEATED", "can\"'t", "hi_`there`",
                              "  leading spaces", "\u20AC", "a\u00E7\u00E3o", "far\u0153", "r.st\u00FCdio:v.1.0.143"),
                            case = "screaming_snake"),
               c("SP_ACE", "REPEATED", "A", "PERCENT", "X", "X_2", "D_9", "REPEATED_2", 
                 "CANT", "HI_THERE", "LEADING_SPACES", "X_3", "ACAO", "FAROE", 
                 "R_STUDIO_V_1_0_143")
  )
  
  expect_equal(clean_names3(c("sp ace", "repeated", "a**^@", "%", "*",
                              "!", "d(!)9", "REPEATED", "can\"'t", "hi_`there`",
                              "  leading spaces", "\u20AC", "a\u00E7\u00E3o", "far\u0153", "r.st\u00FCdio:v.1.0.143"),
                            case = "small_camel"),
               c("spAce", "repeated", "a", "percent", "x", "x_2", "d9", "repeated_2", 
                 "cant", "hiThere", "leadingSpaces", "x_3", "acao", "faroe", "rStudioV1_0_143"
               )
  )
  
  expect_equal(clean_names3(c("sp ace", "repeated", "a**^@", "%", "*",
                              "!", "d(!)9", "REPEATED", "can\"'t", "hi_`there`",
                              "  leading spaces", "\u20AC", "a\u00E7\u00E3o", "far\u0153", "r.st\u00FCdio:v.1.0.143"),
                            case = "big_camel"),
               c("SpAce", "Repeated", "A", "Percent", "X", "X_2", "D9", "Repeated_2", 
                 "Cant", "HiThere", "LeadingSpaces", "X_3", "Acao", "Faroe", "RStudioV1_0_143"
               )
  )
  
  expect_equal(clean_names3(c("sp ace", "repeated", "a**^@", "%", "*",
                              "!", "d(!)9", "REPEATED", "can\"'t", "hi_`there`",
                              "  leading spaces", "\u20AC", "a\u00E7\u00E3o", "far\u0153", "r.st\u00FCdio:v.1.0.143"),
                            case = "lower_upper"),
               c("spACE", "repeated", "a", "percent", "x", "x_2", "d9", "repeated_2", 
                 "cant", "hiTHERE", "leadingSPACES", "x_3", "acao", "faroe", "rSTUDIOv1_0_143"
               )
  )
  
  expect_equal(clean_names3(c("sp ace", "repeated", "a**^@", "%", "*",
                              "!", "d(!)9", "REPEATED", "can\"'t", "hi_`there`",
                              "  leading spaces", "\u20AC", "a\u00E7\u00E3o", "far\u0153", "r.st\u00FCdio:v.1.0.143"),
                            case = "upper_lower"),
               c("SPace", "REPEATED", "A", "PERCENT", "X", "X_2", "D9", "REPEATED_2", 
                 "CANT", "HIthere", "LEADINGspaces", "X_3", "ACAO", "FAROE", "RstudioV1_0_143"
               )
  )
  
  expect_equal(clean_names3(c("sp ace", "repeated", "a**^@", "%", "*",
                              "!", "d(!)9", "REPEATED", "can\"'t", "hi_`there`",
                              "  leading spaces", "\u20AC", "a\u00E7\u00E3o", "far\u0153", "r.st\u00FCdio:v.1.0.143"),
                            case = "mixed"),
               c("sp_ace", "repeated", "a", "percent", "X", "X_2", "d_9", "Repeated", 
                 "cant", "hi_there", "leading_spaces", "X_3", "acao", "faroe", 
                 "r_studio_v_1_0_143")
  )
  
  expect_equal(clean_names3(c("sp ace", "repeated", "a**^@", "%", "*",
                              "!", "d(!)9", "REPEATED", "can\"'t", "hi_`there`",
                              "  leading spaces", "\u20AC", "a\u00E7\u00E3o", "far\u0153", "r.st\u00FCdio:v.1.0.143"),
                            case = "none"),
               c("sp_ace", "repeated", "a", "percent",
                 "X", "X_2", "d_9", "REPEATED",
                 "cant", "hi_there", "leading_spaces", "X_3", 
                 "acao", "faroe", "r_studio_v_1_0_143")
  )
})

# test_that("rules",{
#   examples <- cases[["examples"]]
#   
#   expect_equal(to_snake_case(to_snake_case(examples)),
#                to_snake_case(examples)
#   ) 
#   expect_equal(to_snake_case(to_small_camel_case(examples)),
#                to_snake_case(examples)
#   ) 
#   expect_equal(to_snake_case(to_big_camel_case(examples)),
#                to_snake_case(examples)
#   ) 
#   expect_equal(to_snake_case(to_screaming_snake_case(examples)),
#                to_snake_case(examples)
#   ) 
# })


test_that("random examples", 
          expect_equal(to_any_case("string123", case = "snake"),
                       "string_123"))
# test_that("deprecated",
#           expect_warning(to_any_case("bla", protect = "_"),
#                          "argument protect is deprecated; If you really need this argument, pls submit an issue on https://github.com/Tazinho/snakecase")
#           )

test_that("transliterations", {
  
  skip_if_not( l10n_info()$`UTF-8`)
  
  expect_equal(to_any_case("Älterer Herr", transliterations = c("german", "Herr" = "Mann")), "aelterer_mann")
  
  expect_equal(
    to_any_case("Älterer Herr", transliterations = c( "Herr" = "Mann", "german")),
    "aelterer_mann")
})

test_that("transliterations_error", 
          expect_error(to_any_case("bla", transliterations = "bla"),
                       "Input to `transliterations` must be `NULL`, a string containing elements from the internal lookup dictionaries or from `stringi::stri_trans_list()` or a named vector.",
                       fixed = TRUE))

test_that("empty_fill",
          expect_equal(to_any_case("", empty_fill = "bla"),
          "bla"))

test_that("sentence",
          expect_equal(to_any_case("bla bla_bal", case = "sentence"),
                       "Bla bla bal"))
  
test_that("flip and swap", {
          expect_equal(to_any_case("rSTUDIO", case = "flip"), "Rstudio")
          
          expect_equal(to_any_case("rSTUDIO", case = "swap"), "Rstudio")
})

test_that("complex strings", {
  
  skip_if_not( l10n_info()$`UTF-8`)
  
  strings2 <- c("this - Is_-: a Strange_string", "\u00C4ND THIS ANOTHER_One")
  
  expect_equal(to_any_case(strings2, case = "snake", sep_in = "-|\\:"),
               c("this_is_a_strange_string", "\u00E4nd_this_another_one"))
  
  expect_equal(to_any_case("MERKWUERDIGER-VariablenNAME mit.VIELENMustern_version: 3.7.4",
                           case = "snake",
                           sep_in = "-|:|(?<!\\d)\\.",
                           sep_out = "."),
               "merkwuerdiger.variablen.name.mit.vielen.mustern.version.3.7.4")
  
  expect_equal(to_any_case("R.Studio", case = "big_camel"),
               c("RStudio"))
  
  expect_equal(to_any_case("R.Studio: v 1.0.143", case = "big_camel", sep_in = "\\.", sep_out = "_"),
               "R_Studio:V_1_0_143")
  
  expect_equal(to_any_case("R.aStudio", case = "snake", sep_out = "-"), "r-a-studio")
  expect_equal(to_any_case("R.aStudio", case = "snake", sep_out = "-"), "r-a-studio")
  expect_equal(to_any_case("R.aStudio", case = "big_camel", sep_out = "-"), "R-A-Studio")
  expect_equal(to_any_case("R.aStudio", case = "big_camel", sep_out = "-"), "R-A-Studio")
  expect_equal(to_any_case("R.aStudio", case = "small_camel", sep_out = "-"), "r-A-Studio")
  expect_equal(to_any_case("R.aStudio", case = "small_camel", sep_out = "-"), "r-A-Studio")
  expect_equal(to_any_case("r.aStudio", sep_out = "-", case = "big_camel"), "R-A-Studio")
  
  expect_equal(to_any_case("rStudio", case = "none", prefix = "rrr."),
               "rrr.rStudio")
  
  expect_equal(to_any_case("Rstudio_STudio_sssTTT", case = "mixed"),
               "Rstudio_S_Tudio_sss_Ttt")
  expect_equal(to_any_case("Rstudio_STudio_sssTTT", case = "mixed", parsing_option = 2),
               "Rstudio_St_udio_sss_Ttt")
  
  expect_equal(to_any_case(names(iris), case = "lower_upper", sep_in = "\\.", sep_out = "-"),
               c("sepal-LENGTH", "sepal-WIDTH", "petal-LENGTH", "petal-WIDTH", "species"))
  
  expect_equal(to_any_case("R.aStudio", case = "lower_upper"),
               "rAstudio")
  expect_equal(to_any_case("R.aStudio", case = "upper_lower"),
               "RaSTUDIO")
  
  expect_equal(to_any_case("R.aStudio", case = "lower_upper"),
               "rAstudio")
  expect_equal(to_any_case("R.aStudio", case = "lower_upper", sep_out = "-"),
               "r-A-studio")
  expect_equal(to_any_case("R.aStudio", case = "lower_upper", sep_out = "-"),
               "r-A-studio")
  
  expect_equal(to_any_case("rstudio", case = "all_caps"),
               "RSTUDIO")
  expect_equal(to_any_case("rstudio", case = "upper_camel"),
               "Rstudio")
  expect_equal(to_any_case("rstudio", case = "lower_camel"),
               "rstudio")
  expect_equal(to_any_case("bla rstudio", case = "lower_camel"),
               "blaRstudio")
  
  expect_equal(to_any_case("R.aStudio", case = "parsed", sep_out = "-"),
               "R-a-Studio")
  expect_equal(to_any_case("R.aStudio", case = "small_camel", sep_out = "-"),
               "r-A-Studio")
  expect_equal(to_any_case("R.aStudio", case = "big_camel", sep_out = "-"),
               "R-A-Studio")
  expect_equal(to_any_case("R.aStudio", case = "screaming_snake", sep_out = "-"),
               "R-A-STUDIO")
  expect_equal(to_any_case("R.aStudio", case = "lower_upper", sep_out = "-"),
               "r-A-studio")
  expect_equal(to_any_case("R.aStudio", case = "upper_lower", sep_out = "-"),
               "R-a-STUDIO")
  expect_equal(to_any_case("R.aStudio", case = "none", sep_out = "-"),
               "R-aStudio")
  expect_equal(to_any_case("R.aStudio", case = "mixed", sep_out = "-"),
               "R-a-Studio")
  
  expect_equal(to_any_case(character(0), sep_out = "c"), character())
  
  expect_equal(to_any_case("fdf 1 2", case = "big_camel"),
               "Fdf1_2")
  
  expect_equal(to_any_case("kleines Ha.schen", "screaming_snake"),
               "KLEINES_HA_SCHEN")
  
  expect_equal(to_any_case("kleines Ha.schen", "screaming_snake"),
               "KLEINES_HA_SCHEN")
  
  expect_equal(to_any_case(c("R.aStudio", NA, NA, NA, NA), case = "upper_lower"),
               c("RaSTUDIO", NA, NA, NA, NA))
  
  expect_equal(to_any_case("ac\u00C4o", transliterations = "Latin-ASCII"),
               "ac_ao")
  
  expect_equal(to_any_case("ac\u00C4o", transliterations = c("german", "Latin-ASCII")),
               "ac_aeo")
  #
  expect_equal(to_any_case("\u00C6", transliterations = c("danish", "Latin-ASCII")),
               "ae")
  expect_equal(to_any_case("\u00E6", transliterations = c("danish", "Latin-ASCII")),
               "ae")
  expect_equal(to_any_case("\u00D8", transliterations = c("danish", "Latin-ASCII")),
               "oe")
  expect_equal(to_any_case("\u00F8", transliterations = c("danish", "Latin-ASCII")),
               "oe")
  expect_equal(to_any_case("\u00C5", transliterations = c("danish", "Latin-ASCII")),
               "aa")
  expect_equal(to_any_case("\u00E5", transliterations = c("danish", "Latin-ASCII")),
               "aa")
  
  expect_equal(to_any_case("\u00C6", transliterations = c("finnish", "Latin-ASCII")),
               "a")
  expect_equal(to_any_case("\u00E6", transliterations = c("finnish", "Latin-ASCII")),
               "a")
  expect_equal(to_any_case("\u00D8", transliterations = c("finnish", "Latin-ASCII")),
               "o")
  expect_equal(to_any_case("\u00F8", transliterations = c("finnish", "Latin-ASCII")),
               "o")
  
  expect_equal(to_any_case("\u00C6", transliterations = c("swedish")),
               "a")
  expect_equal(to_any_case("\u00E6", transliterations = c("swedish")),
               "a")
  expect_equal(to_any_case("\u00D8", transliterations = c("swedish")),
               "o")
  expect_equal(to_any_case("\u00F8", transliterations = c("swedish")),
               "o")
  expect_equal(to_any_case("\u00C5", transliterations = c("swedish")),
               "a")
  expect_equal(to_any_case("\u00E5", transliterations = c("swedish")),
               "a")
  
  expect_equal(to_any_case("\u00E6", transliterations = "Latin-ASCII"),
                "ae")
  
  expect_equal(to_any_case("bla.bla", case = "none", sep_in = "\\."),
               "bla_bla")
  
  expect_equal(to_any_case("blaUSABlaGERBlaZDFBla", abbreviations = c("USA", "GER", "ZDF", "BLA"), case = "mixed"),
               "BLA_USA_BLA_GER_BLA_ZDF_BLA")
  
  expect_equal(to_any_case("someUSPeople", abbreviations = "US", case = "mixed", sep_out = " "),
               "some US People")
  
  # expect_equal(to_any_case(c(NA, NA, NA), "lower_upper"),
  #              rep(NA_character_, 3))
  # 
  # expect_equal(to_any_case(c(NA, NA, NA), "upper_lower"),
  #              rep(NA_character_, 3))
  expect_equal(snakecase::to_any_case("blaBla.bla", case = "big_camel", parsing_option = 3),
               "BlaBlaBla")
  
  expect_equal(to_any_case("bla-bla", case = "upper_camel", parsing_option = 3),
               "BlaBla")
  expect_equal(to_any_case("bla.bla", case = "upper_camel", parsing_option = 3),
               "BlaBla")
  expect_equal(to_any_case("bla.bla", case = "upper_camel"),
               "BlaBla")
  expect_equal(to_any_case("a_b_c_d", case = "upper_camel"),
               "ABCD")
  
  expect_equal(to_any_case("some11 21 31numbers_11 21 32_With11 11_diffs11 22 d", case = "upper_camel"),
               "Some11_21_31Numbers11_21_32With11_11Diffs11_22D")
  expect_equal(to_any_case("some11 21 31numbers_11 21 32_With11 11_diffs11 22 d", case = "lower_camel"),
               "some11_21_31Numbers11_21_32With11_11Diffs11_22D")
  expect_equal(to_any_case("some11 21 31numbers_11 21 32_With11 11_diffs11 22 d", case = "upper_lower"),
               "SOME11_21_31numbers11_21_32WITH11_11diffs11_22D")
  expect_equal(to_any_case("some11 21 31numbers_11 21 32_With11 11_diffs11 22 d", case = "lower_upper"),
               "some11_21_31NUMBERS11_21_32with11_11DIFFS11_22d")
  
})


test_that("stackoverflow answers", {
  expect_equal(to_any_case(c("ThisText", "NextText"), case = "snake", sep_out = "."),
               c("this.text", "next.text"))
  
  expect_equal(to_any_case(c("BobDylanUSA",
                             "MikhailGorbachevUSSR",
                             "HelpfulStackOverflowPeople",
                             "IAmATallDrinkOfWater"),
                           case = "parsed",
                           sep_out = " "),
               c("Bob Dylan USA", "Mikhail Gorbachev USSR",
                 "Helpful Stack Overflow People", "I Am A Tall Drink Of Water"))
  
  expect_equal(to_any_case(c("ICUDays","SexCode","MAX_of_MLD","Age.Group"),
                           case = "snake",
                           sep_in = "\\."),
               c("icu_days", "sex_code", "max_of_mld", "age_group")) 

  expect_equal(to_any_case(c("ICUDays","SexCode","MAX_of_MLD","Age.Group"),
                           case = "small_camel",
                           sep_in = "\\."),
               c("icuDays", "sexCode", "maxOfMld", "ageGroup")) 
  
  expect_equal(unlist(strsplit(snakecase::to_parsed_case("thisIsSomeCamelCase"), "_")),
               c("this", "Is", "Some", "Camel", "Case"))
  
  expect_equal(to_any_case(c("zip code", "state", "final count"),
                           case = "big_camel",
                           sep_out = " "),
               c("Zip Code", "State", "Final Count"))
  
  expect_equal(to_any_case(c("this.text", "next.text"),
                           case = "big_camel", 
                           sep_in = "\\."),
               c("ThisText", "NextText"))
})

test_that("parsing cases", {
  expect_equal(to_any_case("RRRStudio", case = "parsed"), 
               "RRR_Studio")
  
  expect_equal(to_any_case("RRRStudio", case = "parsed", parsing_option = 1), 
               "RRR_Studio")

  expect_equal(to_any_case("RRRStudio", case = "parsed", parsing_option = 2), 
               "RRRS_tudio")
  
  expect_equal(check_design_rule("RRRStudio", parsing_option = 1),
               TRUE)
  
  expect_equal(check_design_rule("RRRStudio", parsing_option = 2),
               TRUE)
  })

test_that("expand.grid", {
  # string <- c(NA, "_", "s_na_k_er", "SNAKE SNAKE CASE", "snakeSnakECase",
  #             "SNAKE snakE_case", "ssRRss", "ssRRRR", "thisIsSomeCamelCase",
  #             "this.text", "final count", "BobDylanUSA", "MikhailGorbachevUSSR",
  #             "HelpfulStackOverflowPeople", "ImATallDrinkOfWater", "ICUDays", "SexCode",
  #             "MAX_of_MLD", "Age.Group")
  # case <- c("parsed", "snake", "small_camel", "big_camel", "screaming_snake")
  # prefix <- c("", "start.")
  # postfix <- c("", ".end")
  # replace_special_characters <- c(TRUE, FALSE)
  # 
  # dat <- expand.grid(string = string,
  #                    case = case,
  #                    postfix = postfix,
  #                    prefix = prefix,
  #                    replace_special_characters = replace_special_characters,
  #                    stringsAsFactors = FALSE)
  expect_equal(purrrlyr::invoke_rows(snakecase::to_any_case, dat,
                     sep_in = NULL,
                     sep_out = NULL,
                     .collate = "cols",
                     .to = "output")$output, #%>% dput
               c(NA, "", "s_na_k_er", "SNAKE_SNAKE_CASE", "snake_Snak_E_Case", 
                 "SNAKE_snak_E_case", "ss_R_Rss", "ss_RRRR", "this_Is_Some_Camel_Case", 
                 "this.text", "final_count", "Bob_Dylan_USA", "Mikhail_Gorbachev_USSR", 
                 "Helpful_Stack_Overflow_People", "Im_A_Tall_Drink_Of_Water", 
                 "ICU_Days", "Sex_Code", "MAX_of_MLD", "Age.Group", NA, "", 
                 "s_na_k_er", "snake_snake_case", "snake_snak_e_case", "snake_snak_e_case", 
                 "ss_r_rss", "ss_rrrr", "this_is_some_camel_case", "this.text", 
                 "final_count", "bob_dylan_usa", "mikhail_gorbachev_ussr", "helpful_stack_overflow_people", 
                 "im_a_tall_drink_of_water", "icu_days", "sex_code", "max_of_mld", 
                 "age.group", NA, "", "sNaKEr", "snakeSnakeCase", "snakeSnakECase", 
                 "snakeSnakECase", "ssRRss", "ssRrrr", "thisIsSomeCamelCase", 
                 "this.Text", "finalCount", "bobDylanUsa", "mikhailGorbachevUssr", 
                 "helpfulStackOverflowPeople", "imATallDrinkOfWater", "icuDays", 
                 "sexCode", "maxOfMld", "age.Group", NA, "", "SNaKEr", "SnakeSnakeCase", 
                 "SnakeSnakECase", "SnakeSnakECase", "SsRRss", "SsRrrr", "ThisIsSomeCamelCase", 
                 "This.Text", "FinalCount", "BobDylanUsa", "MikhailGorbachevUssr", 
                 "HelpfulStackOverflowPeople", "ImATallDrinkOfWater", "IcuDays", 
                 "SexCode", "MaxOfMld", "Age.Group", NA, "", "S_NA_K_ER", "SNAKE_SNAKE_CASE", 
                 "SNAKE_SNAK_E_CASE", "SNAKE_SNAK_E_CASE", "SS_R_RSS", "SS_RRRR", 
                 "THIS_IS_SOME_CAMEL_CASE", "THIS.TEXT", "FINAL_COUNT", "BOB_DYLAN_USA", 
                 "MIKHAIL_GORBACHEV_USSR", "HELPFUL_STACK_OVERFLOW_PEOPLE", "IM_A_TALL_DRINK_OF_WATER", 
                 "ICU_DAYS", "SEX_CODE", "MAX_OF_MLD", "AGE.GROUP", NA, ".end", 
                 "s_na_k_er.end", "SNAKE_SNAKE_CASE.end", "snake_Snak_E_Case.end", 
                 "SNAKE_snak_E_case.end", "ss_R_Rss.end", "ss_RRRR.end", "this_Is_Some_Camel_Case.end", 
                 "this.text.end", "final_count.end", "Bob_Dylan_USA.end", "Mikhail_Gorbachev_USSR.end", 
                 "Helpful_Stack_Overflow_People.end", "Im_A_Tall_Drink_Of_Water.end", 
                 "ICU_Days.end", "Sex_Code.end", "MAX_of_MLD.end", "Age.Group.end", 
                 NA, ".end", "s_na_k_er.end", "snake_snake_case.end", "snake_snak_e_case.end", 
                 "snake_snak_e_case.end", "ss_r_rss.end", "ss_rrrr.end", "this_is_some_camel_case.end", 
                 "this.text.end", "final_count.end", "bob_dylan_usa.end", "mikhail_gorbachev_ussr.end", 
                 "helpful_stack_overflow_people.end", "im_a_tall_drink_of_water.end", 
                 "icu_days.end", "sex_code.end", "max_of_mld.end", "age.group.end", 
                 NA, ".end", "sNaKEr.end", "snakeSnakeCase.end", "snakeSnakECase.end", 
                 "snakeSnakECase.end", "ssRRss.end", "ssRrrr.end", "thisIsSomeCamelCase.end", 
                 "this.Text.end", "finalCount.end", "bobDylanUsa.end", "mikhailGorbachevUssr.end", 
                 "helpfulStackOverflowPeople.end", "imATallDrinkOfWater.end", 
                 "icuDays.end", "sexCode.end", "maxOfMld.end", "age.Group.end", 
                 NA, ".end", "SNaKEr.end", "SnakeSnakeCase.end", "SnakeSnakECase.end", 
                 "SnakeSnakECase.end", "SsRRss.end", "SsRrrr.end", "ThisIsSomeCamelCase.end", 
                 "This.Text.end", "FinalCount.end", "BobDylanUsa.end", "MikhailGorbachevUssr.end", 
                 "HelpfulStackOverflowPeople.end", "ImATallDrinkOfWater.end", 
                 "IcuDays.end", "SexCode.end", "MaxOfMld.end", "Age.Group.end", 
                 NA, ".end", "S_NA_K_ER.end", "SNAKE_SNAKE_CASE.end", "SNAKE_SNAK_E_CASE.end", 
                 "SNAKE_SNAK_E_CASE.end", "SS_R_RSS.end", "SS_RRRR.end", "THIS_IS_SOME_CAMEL_CASE.end", 
                 "THIS.TEXT.end", "FINAL_COUNT.end", "BOB_DYLAN_USA.end", "MIKHAIL_GORBACHEV_USSR.end", 
                 "HELPFUL_STACK_OVERFLOW_PEOPLE.end", "IM_A_TALL_DRINK_OF_WATER.end", 
                 "ICU_DAYS.end", "SEX_CODE.end", "MAX_OF_MLD.end", "AGE.GROUP.end", 
                 NA, "start.", "start.s_na_k_er", "start.SNAKE_SNAKE_CASE", "start.snake_Snak_E_Case", 
                 "start.SNAKE_snak_E_case", "start.ss_R_Rss", "start.ss_RRRR", 
                 "start.this_Is_Some_Camel_Case", "start.this.text", "start.final_count", 
                 "start.Bob_Dylan_USA", "start.Mikhail_Gorbachev_USSR", "start.Helpful_Stack_Overflow_People", 
                 "start.Im_A_Tall_Drink_Of_Water", "start.ICU_Days", "start.Sex_Code", 
                 "start.MAX_of_MLD", "start.Age.Group", NA, "start.", "start.s_na_k_er", 
                 "start.snake_snake_case", "start.snake_snak_e_case", "start.snake_snak_e_case", 
                 "start.ss_r_rss", "start.ss_rrrr", "start.this_is_some_camel_case", 
                 "start.this.text", "start.final_count", "start.bob_dylan_usa", 
                 "start.mikhail_gorbachev_ussr", "start.helpful_stack_overflow_people", 
                 "start.im_a_tall_drink_of_water", "start.icu_days", "start.sex_code", 
                 "start.max_of_mld", "start.age.group", NA, "start.", "start.sNaKEr", 
                 "start.snakeSnakeCase", "start.snakeSnakECase", "start.snakeSnakECase", 
                 "start.ssRRss", "start.ssRrrr", "start.thisIsSomeCamelCase", 
                 "start.this.Text", "start.finalCount", "start.bobDylanUsa", "start.mikhailGorbachevUssr", 
                 "start.helpfulStackOverflowPeople", "start.imATallDrinkOfWater", 
                 "start.icuDays", "start.sexCode", "start.maxOfMld", "start.age.Group", 
                 NA, "start.", "start.SNaKEr", "start.SnakeSnakeCase", "start.SnakeSnakECase", 
                 "start.SnakeSnakECase", "start.SsRRss", "start.SsRrrr", "start.ThisIsSomeCamelCase", 
                 "start.This.Text", "start.FinalCount", "start.BobDylanUsa", "start.MikhailGorbachevUssr", 
                 "start.HelpfulStackOverflowPeople", "start.ImATallDrinkOfWater", 
                 "start.IcuDays", "start.SexCode", "start.MaxOfMld", "start.Age.Group", 
                 NA, "start.", "start.S_NA_K_ER", "start.SNAKE_SNAKE_CASE", "start.SNAKE_SNAK_E_CASE", 
                 "start.SNAKE_SNAK_E_CASE", "start.SS_R_RSS", "start.SS_RRRR", 
                 "start.THIS_IS_SOME_CAMEL_CASE", "start.THIS.TEXT", "start.FINAL_COUNT", 
                 "start.BOB_DYLAN_USA", "start.MIKHAIL_GORBACHEV_USSR", "start.HELPFUL_STACK_OVERFLOW_PEOPLE", 
                 "start.IM_A_TALL_DRINK_OF_WATER", "start.ICU_DAYS", "start.SEX_CODE", 
                 "start.MAX_OF_MLD", "start.AGE.GROUP", NA, "start..end", "start.s_na_k_er.end", 
                 "start.SNAKE_SNAKE_CASE.end", "start.snake_Snak_E_Case.end", 
                 "start.SNAKE_snak_E_case.end", "start.ss_R_Rss.end", "start.ss_RRRR.end", 
                 "start.this_Is_Some_Camel_Case.end", "start.this.text.end", 
                 "start.final_count.end", "start.Bob_Dylan_USA.end", "start.Mikhail_Gorbachev_USSR.end", 
                 "start.Helpful_Stack_Overflow_People.end", "start.Im_A_Tall_Drink_Of_Water.end", 
                 "start.ICU_Days.end", "start.Sex_Code.end", "start.MAX_of_MLD.end", 
                 "start.Age.Group.end", NA, "start..end", "start.s_na_k_er.end", 
                 "start.snake_snake_case.end", "start.snake_snak_e_case.end", 
                 "start.snake_snak_e_case.end", "start.ss_r_rss.end", "start.ss_rrrr.end", 
                 "start.this_is_some_camel_case.end", "start.this.text.end", 
                 "start.final_count.end", "start.bob_dylan_usa.end", "start.mikhail_gorbachev_ussr.end", 
                 "start.helpful_stack_overflow_people.end", "start.im_a_tall_drink_of_water.end", 
                 "start.icu_days.end", "start.sex_code.end", "start.max_of_mld.end", 
                 "start.age.group.end", NA, "start..end", "start.sNaKEr.end", 
                 "start.snakeSnakeCase.end", "start.snakeSnakECase.end", "start.snakeSnakECase.end", 
                 "start.ssRRss.end", "start.ssRrrr.end", "start.thisIsSomeCamelCase.end", 
                 "start.this.Text.end", "start.finalCount.end", "start.bobDylanUsa.end", 
                 "start.mikhailGorbachevUssr.end", "start.helpfulStackOverflowPeople.end", 
                 "start.imATallDrinkOfWater.end", "start.icuDays.end", "start.sexCode.end", 
                 "start.maxOfMld.end", "start.age.Group.end", NA, "start..end", 
                 "start.SNaKEr.end", "start.SnakeSnakeCase.end", "start.SnakeSnakECase.end", 
                 "start.SnakeSnakECase.end", "start.SsRRss.end", "start.SsRrrr.end", 
                 "start.ThisIsSomeCamelCase.end", "start.This.Text.end", "start.FinalCount.end", 
                 "start.BobDylanUsa.end", "start.MikhailGorbachevUssr.end", "start.HelpfulStackOverflowPeople.end", 
                 "start.ImATallDrinkOfWater.end", "start.IcuDays.end", "start.SexCode.end", 
                 "start.MaxOfMld.end", "start.Age.Group.end", NA, "start..end", 
                 "start.S_NA_K_ER.end", "start.SNAKE_SNAKE_CASE.end", "start.SNAKE_SNAK_E_CASE.end", 
                 "start.SNAKE_SNAK_E_CASE.end", "start.SS_R_RSS.end", "start.SS_RRRR.end", 
                 "start.THIS_IS_SOME_CAMEL_CASE.end", "start.THIS.TEXT.end", 
                 "start.FINAL_COUNT.end", "start.BOB_DYLAN_USA.end", "start.MIKHAIL_GORBACHEV_USSR.end", 
                 "start.HELPFUL_STACK_OVERFLOW_PEOPLE.end", "start.IM_A_TALL_DRINK_OF_WATER.end", 
                 "start.ICU_DAYS.end", "start.SEX_CODE.end", "start.MAX_OF_MLD.end", 
                 "start.AGE.GROUP.end", NA, "", "s_na_k_er", "SNAKE_SNAKE_CASE", 
                 "snake_Snak_E_Case", "SNAKE_snak_E_case", "ss_R_Rss", "ss_RRRR", 
                 "this_Is_Some_Camel_Case", "this.text", "final_count", "Bob_Dylan_USA", 
                 "Mikhail_Gorbachev_USSR", "Helpful_Stack_Overflow_People", "Im_A_Tall_Drink_Of_Water", 
                 "ICU_Days", "Sex_Code", "MAX_of_MLD", "Age.Group", NA, "", 
                 "s_na_k_er", "snake_snake_case", "snake_snak_e_case", "snake_snak_e_case", 
                 "ss_r_rss", "ss_rrrr", "this_is_some_camel_case", "this.text", 
                 "final_count", "bob_dylan_usa", "mikhail_gorbachev_ussr", "helpful_stack_overflow_people", 
                 "im_a_tall_drink_of_water", "icu_days", "sex_code", "max_of_mld", 
                 "age.group", NA, "", "sNaKEr", "snakeSnakeCase", "snakeSnakECase", 
                 "snakeSnakECase", "ssRRss", "ssRrrr", "thisIsSomeCamelCase", 
                 "this.Text", "finalCount", "bobDylanUsa", "mikhailGorbachevUssr", 
                 "helpfulStackOverflowPeople", "imATallDrinkOfWater", "icuDays", 
                 "sexCode", "maxOfMld", "age.Group", NA, "", "SNaKEr", "SnakeSnakeCase", 
                 "SnakeSnakECase", "SnakeSnakECase", "SsRRss", "SsRrrr", "ThisIsSomeCamelCase", 
                 "This.Text", "FinalCount", "BobDylanUsa", "MikhailGorbachevUssr", 
                 "HelpfulStackOverflowPeople", "ImATallDrinkOfWater", "IcuDays", 
                 "SexCode", "MaxOfMld", "Age.Group", NA, "", "S_NA_K_ER", "SNAKE_SNAKE_CASE", 
                 "SNAKE_SNAK_E_CASE", "SNAKE_SNAK_E_CASE", "SS_R_RSS", "SS_RRRR", 
                 "THIS_IS_SOME_CAMEL_CASE", "THIS.TEXT", "FINAL_COUNT", "BOB_DYLAN_USA", 
                 "MIKHAIL_GORBACHEV_USSR", "HELPFUL_STACK_OVERFLOW_PEOPLE", "IM_A_TALL_DRINK_OF_WATER", 
                 "ICU_DAYS", "SEX_CODE", "MAX_OF_MLD", "AGE.GROUP", NA, ".end", 
                 "s_na_k_er.end", "SNAKE_SNAKE_CASE.end", "snake_Snak_E_Case.end", 
                 "SNAKE_snak_E_case.end", "ss_R_Rss.end", "ss_RRRR.end", "this_Is_Some_Camel_Case.end", 
                 "this.text.end", "final_count.end", "Bob_Dylan_USA.end", "Mikhail_Gorbachev_USSR.end", 
                 "Helpful_Stack_Overflow_People.end", "Im_A_Tall_Drink_Of_Water.end", 
                 "ICU_Days.end", "Sex_Code.end", "MAX_of_MLD.end", "Age.Group.end", 
                 NA, ".end", "s_na_k_er.end", "snake_snake_case.end", "snake_snak_e_case.end", 
                 "snake_snak_e_case.end", "ss_r_rss.end", "ss_rrrr.end", "this_is_some_camel_case.end", 
                 "this.text.end", "final_count.end", "bob_dylan_usa.end", "mikhail_gorbachev_ussr.end", 
                 "helpful_stack_overflow_people.end", "im_a_tall_drink_of_water.end", 
                 "icu_days.end", "sex_code.end", "max_of_mld.end", "age.group.end", 
                 NA, ".end", "sNaKEr.end", "snakeSnakeCase.end", "snakeSnakECase.end", 
                 "snakeSnakECase.end", "ssRRss.end", "ssRrrr.end", "thisIsSomeCamelCase.end", 
                 "this.Text.end", "finalCount.end", "bobDylanUsa.end", "mikhailGorbachevUssr.end", 
                 "helpfulStackOverflowPeople.end", "imATallDrinkOfWater.end", 
                 "icuDays.end", "sexCode.end", "maxOfMld.end", "age.Group.end", 
                 NA, ".end", "SNaKEr.end", "SnakeSnakeCase.end", "SnakeSnakECase.end", 
                 "SnakeSnakECase.end", "SsRRss.end", "SsRrrr.end", "ThisIsSomeCamelCase.end", 
                 "This.Text.end", "FinalCount.end", "BobDylanUsa.end", "MikhailGorbachevUssr.end", 
                 "HelpfulStackOverflowPeople.end", "ImATallDrinkOfWater.end", 
                 "IcuDays.end", "SexCode.end", "MaxOfMld.end", "Age.Group.end", 
                 NA, ".end", "S_NA_K_ER.end", "SNAKE_SNAKE_CASE.end", "SNAKE_SNAK_E_CASE.end", 
                 "SNAKE_SNAK_E_CASE.end", "SS_R_RSS.end", "SS_RRRR.end", "THIS_IS_SOME_CAMEL_CASE.end", 
                 "THIS.TEXT.end", "FINAL_COUNT.end", "BOB_DYLAN_USA.end", "MIKHAIL_GORBACHEV_USSR.end", 
                 "HELPFUL_STACK_OVERFLOW_PEOPLE.end", "IM_A_TALL_DRINK_OF_WATER.end", 
                 "ICU_DAYS.end", "SEX_CODE.end", "MAX_OF_MLD.end", "AGE.GROUP.end", 
                 NA, "start.", "start.s_na_k_er", "start.SNAKE_SNAKE_CASE", "start.snake_Snak_E_Case", 
                 "start.SNAKE_snak_E_case", "start.ss_R_Rss", "start.ss_RRRR", 
                 "start.this_Is_Some_Camel_Case", "start.this.text", "start.final_count", 
                 "start.Bob_Dylan_USA", "start.Mikhail_Gorbachev_USSR", "start.Helpful_Stack_Overflow_People", 
                 "start.Im_A_Tall_Drink_Of_Water", "start.ICU_Days", "start.Sex_Code", 
                 "start.MAX_of_MLD", "start.Age.Group", NA, "start.", "start.s_na_k_er", 
                 "start.snake_snake_case", "start.snake_snak_e_case", "start.snake_snak_e_case", 
                 "start.ss_r_rss", "start.ss_rrrr", "start.this_is_some_camel_case", 
                 "start.this.text", "start.final_count", "start.bob_dylan_usa", 
                 "start.mikhail_gorbachev_ussr", "start.helpful_stack_overflow_people", 
                 "start.im_a_tall_drink_of_water", "start.icu_days", "start.sex_code", 
                 "start.max_of_mld", "start.age.group", NA, "start.", "start.sNaKEr", 
                 "start.snakeSnakeCase", "start.snakeSnakECase", "start.snakeSnakECase", 
                 "start.ssRRss", "start.ssRrrr", "start.thisIsSomeCamelCase", 
                 "start.this.Text", "start.finalCount", "start.bobDylanUsa", "start.mikhailGorbachevUssr", 
                 "start.helpfulStackOverflowPeople", "start.imATallDrinkOfWater", 
                 "start.icuDays", "start.sexCode", "start.maxOfMld", "start.age.Group", 
                 NA, "start.", "start.SNaKEr", "start.SnakeSnakeCase", "start.SnakeSnakECase", 
                 "start.SnakeSnakECase", "start.SsRRss", "start.SsRrrr", "start.ThisIsSomeCamelCase", 
                 "start.This.Text", "start.FinalCount", "start.BobDylanUsa", "start.MikhailGorbachevUssr", 
                 "start.HelpfulStackOverflowPeople", "start.ImATallDrinkOfWater", 
                 "start.IcuDays", "start.SexCode", "start.MaxOfMld", "start.Age.Group", 
                 NA, "start.", "start.S_NA_K_ER", "start.SNAKE_SNAKE_CASE", "start.SNAKE_SNAK_E_CASE", 
                 "start.SNAKE_SNAK_E_CASE", "start.SS_R_RSS", "start.SS_RRRR", 
                 "start.THIS_IS_SOME_CAMEL_CASE", "start.THIS.TEXT", "start.FINAL_COUNT", 
                 "start.BOB_DYLAN_USA", "start.MIKHAIL_GORBACHEV_USSR", "start.HELPFUL_STACK_OVERFLOW_PEOPLE", 
                 "start.IM_A_TALL_DRINK_OF_WATER", "start.ICU_DAYS", "start.SEX_CODE", 
                 "start.MAX_OF_MLD", "start.AGE.GROUP", NA, "start..end", "start.s_na_k_er.end", 
                 "start.SNAKE_SNAKE_CASE.end", "start.snake_Snak_E_Case.end", 
                 "start.SNAKE_snak_E_case.end", "start.ss_R_Rss.end", "start.ss_RRRR.end", 
                 "start.this_Is_Some_Camel_Case.end", "start.this.text.end", 
                 "start.final_count.end", "start.Bob_Dylan_USA.end", "start.Mikhail_Gorbachev_USSR.end", 
                 "start.Helpful_Stack_Overflow_People.end", "start.Im_A_Tall_Drink_Of_Water.end", 
                 "start.ICU_Days.end", "start.Sex_Code.end", "start.MAX_of_MLD.end", 
                 "start.Age.Group.end", NA, "start..end", "start.s_na_k_er.end", 
                 "start.snake_snake_case.end", "start.snake_snak_e_case.end", 
                 "start.snake_snak_e_case.end", "start.ss_r_rss.end", "start.ss_rrrr.end", 
                 "start.this_is_some_camel_case.end", "start.this.text.end", 
                 "start.final_count.end", "start.bob_dylan_usa.end", "start.mikhail_gorbachev_ussr.end", 
                 "start.helpful_stack_overflow_people.end", "start.im_a_tall_drink_of_water.end", 
                 "start.icu_days.end", "start.sex_code.end", "start.max_of_mld.end", 
                 "start.age.group.end", NA, "start..end", "start.sNaKEr.end", 
                 "start.snakeSnakeCase.end", "start.snakeSnakECase.end", "start.snakeSnakECase.end", 
                 "start.ssRRss.end", "start.ssRrrr.end", "start.thisIsSomeCamelCase.end", 
                 "start.this.Text.end", "start.finalCount.end", "start.bobDylanUsa.end", 
                 "start.mikhailGorbachevUssr.end", "start.helpfulStackOverflowPeople.end", 
                 "start.imATallDrinkOfWater.end", "start.icuDays.end", "start.sexCode.end", 
                 "start.maxOfMld.end", "start.age.Group.end", NA, "start..end", 
                 "start.SNaKEr.end", "start.SnakeSnakeCase.end", "start.SnakeSnakECase.end", 
                 "start.SnakeSnakECase.end", "start.SsRRss.end", "start.SsRrrr.end", 
                 "start.ThisIsSomeCamelCase.end", "start.This.Text.end", "start.FinalCount.end", 
                 "start.BobDylanUsa.end", "start.MikhailGorbachevUssr.end", "start.HelpfulStackOverflowPeople.end", 
                 "start.ImATallDrinkOfWater.end", "start.IcuDays.end", "start.SexCode.end", 
                 "start.MaxOfMld.end", "start.Age.Group.end", NA, "start..end", 
                 "start.S_NA_K_ER.end", "start.SNAKE_SNAKE_CASE.end", "start.SNAKE_SNAK_E_CASE.end", 
                 "start.SNAKE_SNAK_E_CASE.end", "start.SS_R_RSS.end", "start.SS_RRRR.end", 
                 "start.THIS_IS_SOME_CAMEL_CASE.end", "start.THIS.TEXT.end", 
                 "start.FINAL_COUNT.end", "start.BOB_DYLAN_USA.end", "start.MIKHAIL_GORBACHEV_USSR.end", 
                 "start.HELPFUL_STACK_OVERFLOW_PEOPLE.end", "start.IM_A_TALL_DRINK_OF_WATER.end", 
                 "start.ICU_DAYS.end", "start.SEX_CODE.end", "start.MAX_OF_MLD.end", 
                 "start.AGE.GROUP.end"))
  }
)



test_that("sep_out", {
  paste_along <- function(x, along = "_") {
    if (length(x) <= 1L) return(x)
    if (length(along) == 1L) return(paste0(x, collapse = along))
    
    along <- c(along, rep_len(along[length(along)], max(length(x) - length(along), 0L)))
    paste0(paste0(x[seq_len(length(x) - 1)], along[seq_len(length(x) - 1)],
                  collapse = ""), x[length(x)])
  }

  expect_equal(to_any_case("a", sep_out = "-"), "a")
  expect_equal(to_any_case(""), "")
  expect_equal(to_any_case("a"), "a")
  expect_equal(to_any_case(c("bla_bla_bla"), sep_out = c("-", "_")),  "bla-bla_bla")
  
  expect_equal(to_any_case(c("2018_01_01_bla_bla_bla"), sep_out = c("-", "_")), "2018-01_01_bla_bla_bla")
  expect_equal(to_any_case(c("2018_01_01_bla_bla_bla"), sep_out = "-"), "2018-01-01-bla-bla-bla")
  expect_equal(to_any_case(c("2018_01_01_bla_bla"), sep_out = c("-", "-", "_", "_", "_", "_", "_")),  "2018-01-01_bla_bla")
  expect_equal(to_any_case(character(0), sep_out = c("_", "_")), character(0))
})

test_that("random case", {
  expect_equal(
    {suppressWarnings(RNGversion("3.1")); set.seed(123); to_any_case("almost RANDOM", case = "random")},
    "AlMosT raNdOm"
    )
})

test_that("title case", {
  expect_equal(
    to_any_case(c("on_andOn", "AndON", " and on", "and so on", "seems like it works", "also abbreviations ETC"), case = "title", abbreviations = "ETC"),
    c("On and on", "And on", "And on", "And so on", "Seems Like it Works", "Also Abbreviations ETC") 
  )
})

test_that("case none", {
  expect_equal(
    to_any_case(c("blabla", "blablub", "blaBlub"),case = "none", transliterations = c(blab = "blub")),
    c("blubla", "blublub", "blaBlub") 
  )
})

test_that("special_input", {
  expect_identical(to_any_case(NA_character_), NA_character_)
  expect_equal(to_any_case(character(0)), character(0))
})

test_that("special_input_2", {
  skip_if(getRversion() < 3.4)
  # atomics
  expect_equal(to_any_case(character()), character())
  expect_error(to_any_case(logical()), "argument is not a character vector", fixed = TRUE)
  expect_error(to_any_case(integer()), "argument is not a character vector", fixed = TRUE)
  expect_error(to_any_case(double()), "argument is not a character vector", fixed = TRUE)
  # data structures
  expect_error(to_any_case(data.frame()), "argument is not a character vector", fixed = TRUE)
  expect_error(to_any_case(list())      , "argument is not a character vector", fixed = TRUE)
  expect_error(to_any_case(matrix())    , "argument is not a character vector", fixed = TRUE)
  # special input or wrong type
  expect_error(to_any_case(NA)    , "argument is not a character vector", fixed = TRUE)
  expect_error(to_any_case(NA_integer_)    , "argument is not a character vector", fixed = TRUE)
  expect_error(to_any_case(NA_real_)    , "argument is not a character vector", fixed = TRUE)
  expect_equal(to_any_case(NA_character_)    , NA_character_)

  expect_error(to_any_case(TRUE)    , "argument is not a character vector", fixed = TRUE)
  expect_error(to_any_case(1.0)    , "argument is not a character vector", fixed = TRUE)
  expect_error(to_any_case(1L)    , "argument is not a character vector", fixed = TRUE)
  expect_equal(to_any_case(c("a", 1L))    , c("a", "1"))
  
  expect_error(to_any_case(NULL)    , "argument is not a character vector", fixed = TRUE)
  expect_error(to_any_case(NaN)    , "argument is not a character vector", fixed = TRUE)
  expect_error(to_any_case(Inf)    , "argument is not a character vector", fixed = TRUE)
  })

test_that("abbreviations", {
  expect_equal(to_any_case("IDENTICALid3", abbreviations = "iD3"), "identical_id3")
})

test_that("parsing_options", {
  
  expect_equal(to_any_case("RRRStudioSStudioStudio", case = "parsed", parsing_option = 1),
               "RRR_Studio_S_Studio_Studio"
               )
  
  expect_equal(to_any_case("RRRStudioSStudioStudio", case = "parsed", parsing_option = -1),
               "RRR_Studio_S_Studio_Studio"
               )
  
  expect_equal(to_any_case("RRRStudioSStudioStudio", case = "parsed", parsing_option = 2),
              "RRRS_tudio_SS_tudio_Studio"
              )
  
  expect_equal(to_any_case("RRRStudioSStudioStudio", case = "parsed", parsing_option = -2),
               "RRRS_tudio_SS_tudio_Studio"
               )
  
  expect_equal(to_any_case("RRRStudioSStudioStudio", case = "parsed", parsing_option = 3),
               "RRRStudio_SStudio_Studio"
               )
  
  expect_equal(to_any_case("RRRStudioSStudioStudio", case = "parsed", parsing_option = -3),
               "RRRStudio_SStudio_Studio"
               )
})

test_that("individual abbreviations", {
  
  expect_equal(
    to_any_case("NBAGame", abbreviations = "NBA", case = "mixed"),
    "NBA_Game"
  )
  
  expect_equal(
    to_any_case("NBAGame", abbreviations = "NBa", case = "mixed"),
    "NBa_Game"
  )
  
  expect_equal(
    to_any_case("NBAGame", abbreviations = "baa", case = "mixed"),
    "Nba_Game"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "MVP", case = "mixed"),
    "Game_MVP"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "MVp", case = "mixed"),
    "Game_MVp"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "mvp", case = "mixed"),
    "Game_mvp"
  )
  
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "mvp", case = "upper_camel"),
    "GameMvp"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "MVP", case = "upper_camel"),
    "GameMVP"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "MVp", case = "upper_camel"),
    "GameMVp"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "mvp", case = "title"),
    "Game Mvp"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "MVP", case = "title"),
    "Game MVP"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "MVp", case = "title"),
    "Game MVp"
  )
  
  expect_equal(
    to_any_case("UserID", abbreviations = "id", case = "title"),
    "User Id"
  )

  expect_equal(
    to_any_case("UserID", abbreviations = "ID", case = "title"),
    "User ID"
  )
  
  expect_equal(
    to_any_case("UserID", abbreviations = "id", case = "upper_camel"),
    "UserId"
  )
  
  expect_equal(
    to_any_case("UserID", abbreviations = "ID", case = "upper_camel"),
    "UserID"
  )
  
  expect_equal(
    to_any_case("UserID", abbreviations = "id", case = "mixed"),
    "User_id"
  )
  
  expect_equal(
    to_any_case("UserID", abbreviations = "ID", case= "mixed"),
    "User_ID"
  )
  
  expect_equal(
    to_any_case("UserID", abbreviations = "Id", case = "mixed"),
    "User_Id"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "mvp", case = "lower_camel"),
    "gameMvp"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "MVP", case = "lower_camel"),
    "gameMVP"
  )

  expect_equal(
    to_any_case("GameMVP", abbreviations = "MVp", case = "lower_camel"),
    "gameMVp"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "GAME", case = "lower_camel"),
    "gameMvp"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "game", case = "lower_camel"),
    "gameMvp"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "gGame", case = "lower_camel"),
    "gameMvp"
  )
  
  expect_equal(
    to_any_case("GameMVP", abbreviations = "Game", case = "lower_camel"),
    "gameMvp"
    )
  
  expect_equal(
    to_any_case("nba_finals_mvp", abbreviations = c("nba", "MVP"), case = "upper_camel"),
    "NbaFinalsMVP"
  )
  
  expect_equal(
    to_any_case("nba_finals_mvp", abbreviations = c("nba", "MVp"), case = "upper_camel"),
    "NbaFinalsMVp"
  )
})

test_that("parsing_option 2", {
  expect_equal(to_any_case("blaBLA", abbreviations = "BLA", parsing_option = 2),
               "bla_bla")
})
test_that("parsing_option 3", {
  expect_equal(to_any_case("blaBLA", abbreviations = "BLA", parsing_option = 3),
               "bla_bla")
})

context("to_parsed_case")

test_that("examples", {
  expect_equal(to_parsed_case(cases[["examples"]]),
               cases[["parsed_case"]])}
)

test_that("rules",{
  examples <- cases[["examples"]]
  # note that to parsed case has a little bit different rules than the other converters
  expect_equal(to_parsed_case(to_parsed_case(examples)),
               to_parsed_case(examples))
  
  expect_equal(to_parsed_case(to_snake_case(examples)),
               to_snake_case(examples)) 
  
  #expect_equal(to_parsed_case(to_small_camel_case(examples)),
  #             to_small_camel_case(examples))
  #
  #expect_equal(to_parsed_case(to_big_camel_case(examples)),
  #             to_big_camel_case(examples))
  
  expect_equal(to_parsed_case(to_screaming_snake_case(examples)),
               to_screaming_snake_case(examples)) 
})

test_that("preserve-name-attribute",{
  labs <- c(a = "abcDEF", b = "bbccEE", c = "TeESt it")
  
  expect_equal(
    to_parsed_case(labs),
    structure(c("abc_DEF", "bbcc_EE", "Te_E_St_it"), .Names = c("a", 
                                                                "b", "c"))
  )
})

context("to_upper_lower_case")

test_that("random stuff", {
  expect_equal(to_upper_lower_case("RStudioRRRStudio"),
               "RstudioRRRstudio")
  
  expect_equal(to_upper_lower_case(c("R.aStudio", NA, NA, NA, NA)),
               c("RaSTUDIO", NA, NA, NA, NA))
  }
)

test_that("preserve-name-attribute",{
  labs <- c(a = "abcDEF", b = "bbccEE", c = "TeESt it")
  
  expect_equal(
    to_any_case(labs, case = "upper_lower"),
    structure(c("ABCdef", "BBCCee", "TEeSTit"), .Names = c("a", "b", 
                                                           "c"))
  )
})

context("to_title_case")

test_that("title case", {
  expect_equal(
    to_title_case(c("on_andOn", "AndON", " and on", "and so on", "seems like it works", "also abbreviations ETC"), abbreviations = "ETC"),
    c("On and on", "And on", "And on", "And so on", "Seems Like it Works", "Also Abbreviations ETC") 
  )
})


test_that("title case and abbreviations", {
  expect_equal(
    to_title_case("so sieht es aus", abbreviations = "ES"),
    "So Sieht ES Aus" 
  )
})

#' General case conversion
#' 
#' Function to convert strings to any case
#'
#' @param string A string (for example names of a data frame).
#' 
#' @param case The desired target case, provided as one of the following:
#' \itemize{
#'  \item{snake_case: \code{"snake"}}
#'  \item{lowerCamel: \code{"lower_camel"} or \code{"small_camel"}}
#'  \item{UpperCamel: \code{"upper_camel"} or \code{"big_camel"}}
#'  \item{ALL_CAPS: \code{"all_caps"} or \code{"screaming_snake"}}
#'  \item{lowerUPPER: \code{"lower_upper"}}
#'  \item{UPPERlower: \code{"upper_lower"}}
#'  \item{Sentence case: \code{"sentence"}}
#'  \item{Title Case: \code{"title"} - This one is basically the same as sentence case, but in addition it is wrapped into \code{tools::toTitleCase} and any \code{abbreviations} are always turned into upper case.}
#'}
#'
#'  There are five "special" cases available:
#' \itemize{
#'  \item{\code{"parsed"}: This case is underlying all other cases. 
#'  Every substring a string consists
#'  of becomes surrounded by an underscore (depending on the \code{parsing_option}).
#'   Underscores at the start and end are trimmed. No lower or 
#'  upper case pattern from the input string are changed.}
#'  \item{\code{"mixed"}: Almost the same as \code{case = "parsed"}. Every letter which is not at the start
#'  or behind an underscore is turned into lowercase. If a substring is set as an abbreviation, it will be turned into upper case.}
#'  \item{\code{"swap"}: Upper case letters will be turned into lower case and vice versa. Also \code{case = "flip"} will work.
#'  Doesn't work with any of the other arguments except \code{unique_sep}, \code{empty_fill}, \code{prefix} and \code{postfix}.}
#'  \item{\code{"random"}: Each letter will be randomly turned into lower or upper case. Doesn't work with any of the other arguments except \code{unique_sep}, \code{empty_fill}, \code{prefix} and \code{postfix}.}
#'  \item{\code{"none"}: Neither parsing nor case conversion occur. This case might be helpful, when
#'  one wants to call the function for the quick usage of the other parameters.
#'  To suppress replacement of spaces to underscores set \code{sep_in = NULL}.
#'  Works with \code{sep_in}, \code{transliterations}, \code{sep_out}, \code{prefix},
#'   \code{postfix},
#'   \code{empty_fill} and \code{unique_sep}.}
#'  \item{\code{"internal_parsing"}: This case is returning the internal parsing
#'  (suppressing the internal protection mechanism), which means that alphanumeric characters will be surrounded by underscores.
#'  It should only be used in very rare use cases and is mainly implemented to showcase the internal workings of \code{to_any_case()}}
#'  }
#' 
#' @param abbreviations character. (Case insensitive) matched abbreviations are surrounded by underscores. In this way, they can get recognized by the parser. This is useful when e.g. \code{parsing_option} 1 is needed for the use case, but some abbreviations but some substrings would require \code{parsing_option} 2. Furthermore, this argument also specifies the formatting of abbreviations in the output for the cases title, mixed, lower and upper camel. E.g. for upper camel the first letter is always in upper case, but when the abbreviation is supplied in upper case, this will also be visible in the output.
#'  
#'  Use this feature with care: One letter abbreviations and abbreviations next to each other are hard to read and also not easy to parse for further processing.
#'  
#' @param sep_in (short for separator input) if character, is interpreted as a
#'  regular expression (wrapped internally into \code{stringr::regex()}). 
#'  The default value is a regular expression that matches any sequence of
#'  non-alphanumeric values. All matches will be replaced by underscores 
#'  (additionally to \code{"_"} and \code{" "}, for which this is always true, even
#'  if \code{NULL} is supplied). These underscores are used internally to split
#'  the strings into substrings and specify the word boundaries.
#'  
#' @param parsing_option An integer that will determine the parsing_option.
#' \itemize{
#'  \item{1: \code{"RRRStudio" -> "RRR_Studio"}}
#'  \item{2: \code{"RRRStudio" -> "RRRS_tudio"}}
#'  \item{3: \code{"RRRStudio" -> "RRRSStudio"}. This will become for example \code{"Rrrstudio"} when we convert to lower camel case.}
#'  \item{-1, -2, -3: These \code{parsing_options}'s will suppress the conversion after non-alphanumeric values.}
#'  \item{0: no parsing}
#'  }
#'
#' @param transliterations A character vector (if not \code{NULL}). The entries of this argument
#' need to be elements of \code{stringi::stri_trans_list()} (like "Latin-ASCII", which is often useful) or names of lookup tables (currently only "german" is supported). In the order of the entries the letters of the input
#'  string will be transliterated via \code{stringi::stri_trans_general()} or replaced via the 
#'  matches of the lookup table. When named character elements are supplied as part of `transliterations`, anything that matches the names is replaced by the corresponding value.

#' You should use this feature with care in case of \code{case = "parsed"}, \code{case = "internal_parsing"} and 
#' \code{case = "none"}, since for upper case letters, which have transliterations/replacements
#'  of length 2, the second letter will be transliterated to lowercase, for example Oe, Ae, Ss, which
#'  might not always be what is intended. In this case you can make usage of the option to supply named elements and specify the transliterations yourself.
#' 
#' @param numerals A character specifying the alignment of numerals (\code{"middle"}, \code{left}, \code{right}, \code{asis} or \code{tight}). I.e. \code{numerals = "left"} ensures that no output separator is in front of a digit.
#' 
#' @param sep_out (short for separator output) String that will be used as separator. The defaults are \code{"_"} 
#' and \code{""}, regarding the specified \code{case}. When \code{length(sep_out) > 1}, the last element of \code{sep_out} gets recycled and separators are incorporated per string according to their order.
#' 
#' @param unique_sep A string. If not \code{NULL}, then duplicated names will get 
#' a suffix integer
#' in the order of their appearance. The suffix is separated by the supplied string
#'  to this argument.
#' 
#' @param empty_fill A string. If it is supplied, then each entry that matches "" will be replaced
#' by the supplied string to this argument.
#' 
#' @param prefix prefix (string).
#'
#' @param postfix postfix (string).
#'
#' @return A character vector according the specified parameters above.
#'
#' @note \code{to_any_case()} is vectorised over \code{string}, \code{sep_in}, \code{sep_out},
#'  \code{empty_fill}, \code{prefix} and \code{postfix}.
#'  
#' @author Malte Grosser, \email{malte.grosser@@gmail.com}
#' @keywords utilities
#'
#' @examples
#' ### abbreviations
#' to_snake_case(c("HHcity", "newUSElections"), abbreviations = c("HH", "US"))
#' to_upper_camel_case("succesfullGMBH", abbreviations = "GmbH")
#' to_title_case("succesfullGMBH", abbreviations = "GmbH")
#' 
#' ### sep_in (input separator)
#' string <- "R.St\u00FCdio: v.1.0.143"
#' to_any_case(string)
#' to_any_case(string, sep_in = ":|\\.")
#' to_any_case(string, sep_in = ":|(?<!\\d)\\.")
#'             
#' ### parsing_option
#' # the default option makes no sense in this setting
#' to_parsed_case("HAMBURGcity", parsing_option = 1)
#' # so the second parsing option is the way to address this example
#' to_parsed_case("HAMBURGcity", parsing_option = 2)
#' # By default (option 1) characters are converted after non alpha numeric characters.
#' # To suppress this behaviour add a minus to the parsing_option
#' to_upper_camel_case("lookBehindThe.dot", parsing_option = -1)
#' # For some exotic cases parsing option 3 might be of interest
#' to_parsed_case("PARSingOption3", parsing_option = 3)
#' # There may be reasons to suppress the parsing
#' to_any_case("HAMBURGcity", parsing_option = 0)
#' 
#' ### transliterations
#' to_any_case("\u00E4ngstlicher Has\u00EA", transliterations = c("german", "Latin-ASCII"))
#' 
#' ### case
#' strings <- c("this Is a Strange_string", "AND THIS ANOTHER_One")
#' to_any_case(strings, case = "snake")
#' to_any_case(strings, case = "lower_camel") # same as "small_camel"
#' to_any_case(strings, case = "upper_camel") # same as "big_camel"
#' to_any_case(strings, case = "all_caps") # same as "screaming_snake"
#' to_any_case(strings, case = "lower_upper")
#' to_any_case(strings, case = "upper_lower")
#' to_any_case(strings, case = "sentence")
#' to_any_case(strings, case = "title")
#' to_any_case(strings, case = "parsed")
#' to_any_case(strings, case = "mixed")
#' to_any_case(strings, case = "swap")
#' to_any_case(strings, case = "random")
#' to_any_case(strings, case = "none")
#' to_any_case(strings, case = "internal_parsing")
#' 
#' ### numerals
#' to_snake_case("species42value 23month 7-8", numerals = "asis")
#' to_snake_case("species42value 23month 7-8", numerals = "left")
#' to_snake_case("species42value 23month 7-8", numerals = "right")
#' to_snake_case("species42value 23month 7-8", numerals = "middle")
#' to_snake_case("species42value 23month 7-8", numerals = "tight")
#' 
#' ### sep_out (output separator)
#' string <- c("lowerCamelCase", "ALL_CAPS", "I-DontKNOWWhat_thisCASE_is")
#' to_snake_case(string, sep_out = ".")
#' to_mixed_case(string, sep_out = " ")
#' to_screaming_snake_case(string, sep_out = "=")
#' 
#' ### empty_fill
#' to_any_case(c("","",""), empty_fill = c("empty", "empty", "also empty"))
#' 
#' ### unique_sep
#' to_any_case(c("same", "same", "same", "other"), unique_sep = c(">"))
#' 
#' ### prefix and postfix
#' to_upper_camel_case("some_path", sep_out = "//", 
#'   prefix = "USER://", postfix = ".exe")
#'
#' @seealso \href{https://github.com/Tazinho/snakecase}{snakecase on github} or 
#' \code{\link{caseconverter}} for some handy shortcuts.
#'
#' @export
#'
to_any_case <- function(string,
                        case = c("snake", "small_camel", "big_camel", "screaming_snake", 
                                 "parsed", "mixed", "lower_upper", "upper_lower", "swap",
                                 "all_caps", "lower_camel", "upper_camel", "internal_parsing", 
                                 "none", "flip", "sentence", "random", "title"),
                        abbreviations = NULL,
                        sep_in = "[^[:alnum:]]",
                        parsing_option = 1,
                        transliterations = NULL,
                        numerals = c("middle", "left", "right", "asis", "tight"),
                        sep_out = NULL,
                        unique_sep = NULL,
                        empty_fill = NULL,
                        prefix = "",
                        postfix = ""){
### ____________________________________________________________________________
### Argument matching
  case <- match.arg(case)
  numerals <- match.arg(numerals)
### ____________________________________________________________________________
### Set encoding to utf8
  string <- enc2utf8(string)
### ____________________________________________________________________________
### Argument checking (check input length -> necessary for NULL and atomic(0))
  if (identical(stringr::str_length(string), integer())) {return(character())}
### ____________________________________________________________________________
### Save attributes
  string_attributes <- attributes(string)
### ____________________________________________________________________________
### Handle aliases
  case[case == "all_caps"] <- "screaming_snake"
  case[case == "lower_camel"] <- "small_camel"
  case[case == "upper_camel"] <- "big_camel"
  case[case == "flip"] <- "swap"
### ____________________________________________________________________________
### Prepare abbreviations
  if(!is.null(abbreviations)) {
    abbreviations <- abbreviations[!is.na(abbreviations)]
    abbreviations <- unique(abbreviations)
    names(abbreviations) <- stringr::str_to_lower(abbreviations)
  }
### ____________________________________________________________________________
### Prepare title case
  title <- if (case == "title") TRUE else FALSE
  case[case == "title"] <- "sentence"
### ____________________________________________________________________________
### Handle swap case
  if (case == "swap") {
    string <- gsub(pattern = "([[:upper:]])|([[:lower:]])",
                   perl = TRUE,
                   replacement = "\\L\\1\\U\\2",
                   string)}
### ____________________________________________________________________________
### Handle random case
  if (case == "random") {
    random_case <- function(string) {
      upper_or_lower <- function(string) {
        if(sample(c(TRUE, FALSE), 1)) {return(stringr::str_to_upper(string))}
        stringr::str_to_lower(string)
      }
      
      unlist(
        lapply(
          strsplit(string, split = character(0)),
          function(x) paste0(unlist(lapply(x, upper_or_lower)), collapse = "")
        )
      )
    }
    
    string <- random_case(string)
  }
### ____________________________________________________________________________
### Match abbreviations
  # mark abbreviation by placing an underscore behind them (in front of the parsing)
  if (!case %in% c("swap", "random", "none")) {
    string <- stringr::str_replace_all(string, "[:blank:]", "_") # important, as I'd like to use sth like "_ l abbr r_" around abbreviations
    string <- abbreviation_internal(string, abbreviations)
  }
### ____________________________________________________________________________
### Preprocessing:
## Turn mateches of `sep_in` into "_" and
## surround matches of the parsing by "_" (parsed_case)
  if (!case %in% c("swap", "random")) {
    
    if(case == "none") {
      string <- preprocess_internal(string, sep_in = sep_in)
    }
    
    if (case != "none") {
      string <- to_parsed_case_internal(string,
                                        parsing_option = parsing_option,
                                        numerals = numerals,
                                        abbreviations = abbreviations,
                                        sep_in = sep_in)
    } else {
      string <- vapply(string, stringr::str_replace_all, "","_+", "_", 
                       USE.NAMES = FALSE) 
      string <- vapply(string, stringr::str_replace_all, "","^_|_$", "",
                       USE.NAMES = FALSE)
    }
### ____________________________________________________________________________
### "mixed", "snake", "small_camel", "big_camel", "screaming_case", "parsed"
  if (case %in% c("mixed", "snake", "small_camel",
                 "big_camel", "screaming_snake", "parsed",
                 "lower_upper", "upper_lower", "sentence")) {
### split-----------------------------------------------------------------------
    if (case %in% c("mixed", "snake", "screaming_snake", "parsed", "lower_upper", "upper_lower", "sentence")) {
      string <- stringr::str_split(string, "_")
    }
    #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    if (case %in% c("small_camel", "big_camel")) {
      string <- stringr::str_split(string, pattern = "(?<!\\d)_|_(?!\\d)")
    }
### replacement of special characters_------------------------------------------
    if (!is.null(transliterations)) {
      string <- lapply(string, function(x)
        replace_special_characters_internal(x, transliterations, case))
      string <- lapply(string, function(x) 
        stringr::str_replace_all(x, "_+", "_"))
    }
### caseconversion--------------------------------------------------------------
    if (case == "mixed") {
      # Was muss passieren? Es muss in den no part ein Vektor derselben länge aber mit den Abbreviations.
      # x[abbreviations %in% x]
      string <- lapply(string,
                       function(x) ifelse(!stringr::str_to_lower(x) %in% stringr::str_to_lower(abbreviations), 
                           stringr::str_c(stringr::str_sub(x, 1, 1),
                                          stringr::str_to_lower(stringr::str_sub(x, 2))),
                           abbreviations[stringr::str_to_lower(x)])
        )
      }
    #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    if (case %in% c("snake", "sentence")) {
      string <- lapply(string, stringr::str_to_lower)
    }
    #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    if (case == "big_camel") {
      string <- lapply(string, stringr::str_to_lower)
      if (!is.null(abbreviations)) {
      string <- lapply(string, 
                       function(x) ifelse(stringr::str_to_lower(x) %in% stringr::str_to_lower(abbreviations), 
                                          abbreviations[stringr::str_to_lower(x)],
                                          x)
      )
      }
      string <- lapply(string,
                       function(x) stringr::str_c(stringr::str_to_upper(stringr::str_sub(x, 1, 1)),
                                                  stringr::str_sub(x, 2)))
    }
    #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    if (case == "small_camel") {
      string <- lapply(string, stringr::str_to_lower)
      
      if (!is.null(abbreviations)) { # handle abbreviations
        string <- lapply(string,
                         function(x) c(x[1], 
                                       ifelse(x[-1] %in% stringr::str_to_lower(abbreviations),
                                              abbreviations[x[-1]], x[-1])))
      }
      
      string <- lapply(string, 
                       function(x) stringr::str_c(stringr::str_to_upper(stringr::str_sub(x, 1, 1)),
                                    stringr::str_sub(x, 2)))
      string <- vapply(string, 
                       stringr::str_c, "", collapse = " ",
                       USE.NAMES = FALSE)
      string <- vapply(string, 
                       function(x) stringr::str_c(stringr::str_to_lower(stringr::str_sub(x, 1, 1)),
                                        stringr::str_sub(x, 2)), "",
               USE.NAMES = FALSE)
      string <- stringr::str_split(string, " ")
    }
    #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    if (case == "screaming_snake") {
      string <- lapply(string, stringr::str_to_upper)
    }
    #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    if (case == "lower_upper") {
      string[!is.na(string)] <- mapply(function(x, y) { 
        # odds to lower
        x[y] <- stringr::str_to_lower(x[y]);
        # others to upper
        x[!y] <- stringr::str_to_upper(x[!y]);
        x},
        string[!is.na(string)],
        lapply(string[!is.na(string)], relevant),
        SIMPLIFY = FALSE,
        USE.NAMES = TRUE)
    }
    #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    if (case == "upper_lower") {
      string[!is.na(string)] <- mapply(function(x, y) {
        # odds to upper
        x[y] <- stringr::str_to_upper(x[y]);
        # others to lower
        x[!y] <- stringr::str_to_lower(x[!y]);
        x},
        string[!is.na(string)],
        lapply(string[!is.na(string)], relevant),
        SIMPLIFY = FALSE,
        USE.NAMES = TRUE)
    }
### collapsing------------------------------------------------------------------
    if (case %in% c("snake", "none", "parsed", "mixed", "screaming_snake", 
                   "small_camel", "big_camel", "lower_upper", "upper_lower")) {
      string <- vapply(string, 
                       function(x) stringr::str_c(x, collapse = "_"), "",
                       USE.NAMES = FALSE)
    }
    
    if (case == "sentence" & title){
      
      if (!is.null(abbreviations)) {
        string <- lapply(string, 
                         function(x) ifelse(stringr::str_to_lower(x) %in% stringr::str_to_lower(abbreviations), 
                                            abbreviations[stringr::str_to_lower(x)],
                                            x))
      }
      
      string <- vapply(string, 
                       function(x) stringr::str_c(x, collapse = " "), "",
                       USE.NAMES = FALSE)
      string <- tools::toTitleCase(string)
      string <- stringr::str_replace_all(string, " ", "_")
      
      string <- vapply(string,
                       function(x) stringr::str_c(stringr::str_to_upper(stringr::str_sub(x, 1, 1)),
                                                  stringr::str_sub(x, 2)), "",
                       USE.NAMES = FALSE)
    }
    
    if (case == "sentence") {
      string <- vapply(string, 
                       function(x) stringr::str_c(x, collapse = "_"), "",
                       USE.NAMES = FALSE)
    }
    #. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
    # Protect (only internal, not via an argument).
    # Replace all "_" by "" which are around a not alphanumeric character
    if (numerals == "asis") {
      string <- stringr::str_replace_all(string, " ", "")
    }
    if (numerals == "right" | numerals == "tight") {
      # underscore with a digit before and no digit after
      string <- stringr::str_replace_all(string, "(?<=\\d)_(?!\\d)", "")
    }
    
    if (numerals == "left" | numerals == "tight") {
      # underscore with no digit before and a digit after
      string <- stringr::str_replace_all(string, "(?<!\\d)_(?=\\d)", "")
    }
    
    # omit (old) protection mode
    if (case != "internal_parsing") {
      string <- stringr::str_replace_all(string, "_(?![:alnum:])|(?<![:alnum:])_", "")
    }
### ----------------------------------------------------------------------------
}
### postprocessing (ouput separator)s--------------------------------------------
    # if (!is.null(sep_out) & !identical(string, character(0))) {
    #   string <- mapply(function(x, y) 
    #     stringr::str_replace_all(x, "_", y),
    #                              string,
    #                              sep_out,
    #     USE.NAMES = FALSE)
    #   }
    if (!is.null(sep_out) & !identical(string, character(0))) {
      paste_along <- function(x, along = "_") {
        if (length(x) <= 1L) return(x)
        if (length(along) == 1L) return(paste0(x, collapse = along))
        
        along <- c(along, rep_len(along[length(along)], max(length(x) - length(along), 0L)))
        paste0(paste0(x[seq_len(length(x) - 1)], along[seq_len(length(x) - 1)] ,
                      collapse = ""), x[length(x)])
      }
      
      string <- stringr::str_split(string, pattern = "_")
      string <- vapply(string, 
                       function(x) paste_along(x, along = sep_out), "",
                       USE.NAMES = FALSE)
    }
    
    if (is.null(sep_out) & case %in% c("small_camel", "big_camel", 
                                            "lower_upper", "upper_lower")) {
      string <- stringr::str_replace_all(string, "(?<!\\d)_|_(?!\\d)", "")
    }
    
    if (is.null(sep_out) & (case == "sentence" | (case == "snake" & title))) {
      string <- stringr::str_replace_all(string, "_", " ")
    }

    if (case == "sentence") {
      string <- vapply(string, 
                       function(x) stringr::str_c(stringr::str_to_upper(stringr::str_sub(x, 1, 1)),
                                                  stringr::str_sub(x, 2)), "",
                       USE.NAMES = FALSE)
    }
### ____________________________________________________________________________
### "none"
  if (case == "none" & !is.null(transliterations)) {
    string <- replace_special_characters_internal(string, transliterations, case = case)
  }
### ____________________________________________________________________________    
} # close swap
### ____________________________________________________________________________
### fill empty strings
  if (!is.null(empty_fill) & any(string == "")) {
    string[string == ""] <- empty_fill
  }
### ____________________________________________________________________________
### make unique
  if (!is.null(unique_sep)) {
    string <- make.unique(string, sep = unique_sep)
  }
### ____________________________________________________________________________
### pre and postfix
  string <- stringr::str_c(prefix, string, postfix)
### ____________________________________________________________________________
### set back attributes
  attributes(string) <- string_attributes
### ____________________________________________________________________________
### guarantee that output is really UTF-8
  string <- enc2utf8(string)
### ____________________________________________________________________________
### return
  string
}

#' Internal helper for "lower_upper", "upper_lower".
#' This helper returns a logical vector with TRUE for the 
#' first and every second string of those which contain 
#' an alphabetic character
#'
#' @param string A string (for example names of a data frame).
#' 
#' @return A logical vector.
#'
#' @author Malte Grosser, \email{malte.grosser@@gmail.com}
#' @keywords utilities
#'

relevant <- function(string){
    relevant <- stringr::str_detect(string, "[:alpha:]")
    relevant[relevant] <- rep_len(c(TRUE, FALSE), sum(relevant))
    relevant
}

#' Internal parser, which is relevant for preprocessing, parsing and parsing options
#'
#' @param string A string.
#' @param parsing_option An integer that will determine the parsing option.
#' \itemize{
#'  \item{1: \code{RRRStudio -> RRR_Studio}}
#'  \item{2: \code{RRRStudio -> RRRS_tudio}}
#'  \item{3: parses like option 1 but suppresses "_" around non alpha-numeric characters. In this way this option suppresses splits and resulting case conversion after these characters.}
#'  \item{any other integer <= 0: no parsing"}
#'  }
#' @param numerals A character specifying the alignment of numerals (\code{"middle"}, \code{left}, \code{right} or \code{asis}). I.e. \code{numerals = "left"} ensures that no output separator is in front of a digit.
#' @param abbreviations A character string specifying abbreviations that should be marked to be recognized by later parsing.
#' @param sep_in A character (regular expression) used to specify input separators.
#'  
#' @return A character vector separated by underscores, containing the parsed string.
#'
#' @author Malte Grosser, \email{malte.grosser@@gmail.com}
#' @keywords utilities
#'
to_parsed_case_internal <- function(string, parsing_option = 1L, numerals, abbreviations, sep_in){
  ### input checking
  if (parsing_option >= 4L | parsing_option <= -4) {
    stop("parsing_option must be between -4 and +4.", call. = FALSE)
  }

  ### applying parsing functions  
  # case: 1 RRRStudioSStudioStudio -> RRR_Studio_S_Studio_Studio
  parsing_option_1 <- function(string, numerals, sep_in) {
    string <- preprocess_internal(string, sep_in)

    if (numerals == "asis") {
      string <- parse6_mark_digits(string)
    }
    string <- parse1_pat_cap_smalls(string) # RStudio.V11 -> R_Studio_.V11
    string <- parse2_pat_digits(string)     # RStudio.V11 -> RStudio.V_11_
    string <- parse3_pat_caps(string)       # RStudio.V11 -> _RS_tudio.V11
    string <- parse4_pat_cap(string)        # RStudio.V11 -> RStudio._V_11
    string <- parse5_pat_non_alnums(string) # RStudio.V11 -> RStudio_._V11
    string
  }
  parsing_option_1_abbr <- function(string, numerals, sep_in) {
    string <- stringr::str_split(string, "_\\sl|r\\s_")
    string <- lapply(string,
                     function(x) ifelse(stringr::str_detect(x, "^[^\\sl]"),
                                        parsing_option_1(x, numerals = numerals, sep_in = sep_in), 
                                        x)
                     )
    string <- vapply(string, 
                     function(x) stringr::str_c(x, collapse = "_"), "",
                     USE.NAMES = FALSE)
    string <- stringr::str_replace_all(string, "\\sl|r\\s", "")
    string
  }
  
  if ((parsing_option == 1 | parsing_option == -1) & is.null(abbreviations)) {
    string <- parsing_option_1(string = string, numerals = numerals, sep_in = sep_in)
  }
  if ((parsing_option == 1 | parsing_option == -1) & !is.null(abbreviations)) {
    string_index <- stringr::str_detect(string, pattern = "\\s")
    string[string_index] <- parsing_option_1_abbr(string = string[string_index], numerals = numerals, sep_in = sep_in)
    string[!string_index] <- parsing_option_1(string = string[!string_index], numerals = numerals, sep_in = sep_in)
  }
  # case: 2 RRRStudioSStudioStudio -> RRRS_tudio_SS_tudio_Studio
  parsing_option_2 <- function(string, numerals, sep_in) {
    string <- preprocess_internal(string, sep_in)
    
    if (numerals == "asis") {
      string <- parse6_mark_digits(string)
    }
    string <- parse3_pat_caps(string)
    string <- parse1_pat_cap_smalls(string)
    string <- parse2_pat_digits(string)
    string <- parse4_pat_cap(string)
    string <- parse5_pat_non_alnums(string)
    string
  }
  parsing_option_2_abbr <- function(string, numerals, sep_in) {
    string <- stringr::str_split(string, "_\\sl|r\\s_")
    string <- lapply(string,
                     function(x) ifelse(stringr::str_detect(x, "^[^\\sl]"),
                                        parsing_option_2(x, numerals = numerals, sep_in = sep_in), 
                                        x)
    )
    string <- vapply(string, 
                     function(x) stringr::str_c(x, collapse = "_"), "",
                     USE.NAMES = FALSE)
    string <- stringr::str_replace_all(string, "\\sl|r\\s", "")
    string
  }
  
  if ((parsing_option == 2 | parsing_option == -2) & is.null(abbreviations)) {
    string <- parsing_option_2(string = string, numerals = numerals, sep_in = sep_in)
  }
  if ((parsing_option == 2 | parsing_option == -2) & !is.null(abbreviations)) {
    string_index <- stringr::str_detect(string, pattern = "\\s")
    string[string_index] <- parsing_option_2_abbr(string = string[string_index], numerals = numerals, sep_in = sep_in)
    string[!string_index] <- parsing_option_2(string = string[!string_index], numerals = numerals, sep_in = sep_in)
  }
  # case: 3 RRRStudioSStudioStudio -> RRRStudio_SStudio_Studio
  parsing_option_3 <- function(string, numerals, sep_in) {
    string <- preprocess_internal(string, sep_in)
    
    if (numerals == "asis") {
      string <- parse6_mark_digits(string)
    }
    string <- parse7_pat_caps_smalls(string)
    string <- parse8_pat_smalls_after_non_alnums(string)
    string <- parse2_pat_digits(string)
    string <- parse4_pat_cap(string)
    string <- parse5_pat_non_alnums(string)
    string
  }
  parsing_option_3_abbr <- function(string, numerals, sep_in) {
    string <- stringr::str_split(string, "_\\sl|r\\s_")
    string <- lapply(string,
                     function(x) ifelse(stringr::str_detect(x, "^[^\\sl]"),
                                        parsing_option_3(x, numerals = numerals, sep_in = sep_in), 
                                        x)
    )
    string <- vapply(string, 
                     function(x) stringr::str_c(x, collapse = "_"), "",
                     USE.NAMES = FALSE)
    string <- stringr::str_replace_all(string, "\\sl|r\\s", "")
    string
  }
  
  if ((parsing_option == 3 | parsing_option == -3) & is.null(abbreviations)) {
    string <- parsing_option_3(string, numerals, sep_in = sep_in)
  }
  if ((parsing_option == 3 | parsing_option == -3) & !is.null(abbreviations)) {
    string_index <- stringr::str_detect(string, pattern = "\\s")
    string[string_index] <- parsing_option_3_abbr(string = string[string_index], numerals = numerals, sep_in = sep_in)
    string[!string_index] <- parsing_option_3(string = string[!string_index], numerals = numerals, sep_in = sep_in)
    
  }
  # case:6 email1_2 -> email 1_2
  # if (parsing_option == 4) {
  #   string <- parse5_mark_digits(string)
  #   string <- parse1_pat_cap_smalls(string)
  #   string <- parse2_pat_caps2(string)
  #   string <- parse3_pat_cap_lonely(string)
  #   string <- parse4_separate_non_alnums(string)
  # }
  
  ### customize the output
  # remove more than one "_" and starting/ending "_"
  string <- stringr::str_replace_all(string, "_+", "_")

  string <- stringr::str_replace_all(string, "^_|_$", "")
  
  if (parsing_option %in% c(-1, -2, -3)) {
    string <- stringr::str_replace_all(string, "_(?![:alnum:])|(?<![:alnum:])_", "")
  }
  # return
  string
}

context("to_parsed_case_internal")

test_that("condition 6", {
  expect_error(to_parsed_case_internal("bla", parsing_option = 7),
               "parsing_option must be between -4 and +4.", fixed = TRUE)}
)

test_that("parsing_option -1", {
  expect_equal(to_parsed_case_internal("look_AfterThe-hyphen andThe.dot", parsing_option = -1, numerals = "asis", abbreviations = NULL, sep_in = NULL),
               "look_After_The-hyphen and_The.dot")}
)

test_that("all", {
  expect_equal(to_parsed_case_internal(string  = "bla", numerals = "asis", parsing_option = 2, sep = "_", abbreviations = NULL),
               "bla")
  }
)

test_that("split", {
  expect_equal(to_parsed_case_internal("bla_ lbla", abbreviations = NULL, sep_in =NULL, numerals = "middle")
               ,
               "bla_ _lbla")
  }
)

#' Internal abbreviation marker, marks abbreviations with an underscore behind.
#' Useful if \code{parsing_option} 1 is needed, but some abbreviations need \code{parsing_option} 2.
#'
#' @param string A string (for example names of a data frame).
#' @param abbreviations character with (uppercase) abbreviations. This marks
#'  abbreviations with an underscore behind (in front of the parsing).
#'  Useful if \code{parsing_option} 1 is needed, but some abbreviations need \code{parsing_option} 2.
#' 
#' @return A character vector.
#'
#' @author Malte Grosser, \email{malte.grosser@@gmail.com}
#' @keywords utilities
#'
abbreviation_internal <- function(string, abbreviations = NULL){

  if (!is.null(abbreviations)) {
    abbreviations_upper <- stringr::str_to_upper(abbreviations)
    
    if (length(abbreviations_upper)) {
      # replace at start
      pattern_start <- stringr::str_c("(", stringr::str_c("^", abbreviations_upper, collapse = "|"), ")", "(?=[^[:upper:]])")
      string <- stringr::str_replace_all(string, pattern_start, replacement = "_ l l\\1r r _")
      # replace in the middle
      pattern_middle <- stringr::str_c("(?<=[^[:upper:]])(", stringr::str_c(abbreviations_upper, collapse = "|"), ")", "(?=[^[:upper:]])")
      string <- stringr::str_replace_all(string, pattern_middle, replacement = "_ l l\\1r r _")
      # replace in the end
      pattern_end <- stringr::str_c("(?<=[^[:upper:]])(", stringr::str_c(abbreviations_upper, "$", collapse = "|"), ")")
      string <- stringr::str_replace_all(string, pattern_end, replacement = "_ l l\\1r r _")
      # replace from start to end
      pattern_start_to_end <- stringr::str_c("^(", stringr::str_c(abbreviations_upper, "$", collapse = "|"), ")")
      string <- stringr::str_replace_all(string, pattern_start_to_end, replacement = "_ l l\\1r r _")
    }
    
    abbreviations_lower <- stringr::str_to_lower(abbreviations)
    if (length(abbreviations_lower)) {
      # replace at start
      pattern_start <- stringr::str_c("(", stringr::str_c("^", abbreviations_lower, collapse = "|"), ")", "(?=[^[:lower:]])")
      string <- stringr::str_replace_all(string, pattern_start, replacement = "_ l l\\1r r _")
      # replace in the middle
      pattern_middle <- stringr::str_c("(?<=[^[:lower:]])(", stringr::str_c(abbreviations_lower, collapse = "|"), ")", "(?=[^[:lower:]])")
      string <- stringr::str_replace_all(string, pattern_middle, replacement = "_ l l\\1r r _")
      # replace in the end
      pattern_end <- stringr::str_c("(?<=[^[:lower:]])(", stringr::str_c(abbreviations_lower, "$", collapse = "|"), ")")
      string <- stringr::str_replace_all(string, pattern_end, replacement = "_ l l\\1r r _")
      # replace from start to end
      pattern_start_to_end <- stringr::str_c("^(", stringr::str_c(abbreviations_lower, "$", collapse = "|"), ")")
      string <- stringr::str_replace_all(string, pattern_start_to_end, replacement = "_ l l\\1r r _")
    }
  }
  
  string <- stringr::str_replace_all(string, "(_\\sl\\sl)+", "_ l l")
  # string <- stringr::str_replace_all("_ l l_ l lRSSr r _r r _feed_ l lRSSr r _feed", "(_\\sl\\sl)+", "_ l l")
  string <- stringr::str_replace_all(string, "(r\\sr\\s_)+", "r r _")
  string
}

#' Internal function that replaces regex matches with underscores
#'
#' @param string A string.
#' @param sep_in (short for separator input) A regex supplied as a character (if not \code{NULL}), which will be wrapped internally
#' into \code{stringr::regex()}. All matches will be replaced by underscores (additionally to 
#' \code{"_"} and \code{" "}, for which this is always true). Underscores can later turned into another separator via \code{postprocess}.
#' 
#' @return A character containing the parsed string.
#'
#' @author Malte Grosser, \email{malte.grosser@@gmail.com}
#' @keywords utilities
#'
preprocess_internal <- function(string, sep_in){
  if(!is.null(sep_in)){
    string <- stringr::str_replace_all(string, sep_in, "_")
  }
  string
}

#' Internal helper to replace special characters.
#'
#' @param string A string (for example names of a data frame).
#' @param transliterations A character vector (if not \code{NULL}). The entries of this argument
#' need to be elements of \code{stringi::stri_trans_list()} (like "Latin-ASCII", which is often useful) or names of lookup tables (currently
#' only "german" is supported). In the order of the entries the letters of the input
#'  string will be transliterated via \code{stringi::stri_trans_general()} or replaced via the 
#'  matches of the lookup table. When named character elements are supplied as part of `transliterations`, anything that matches the names is replaced by the corresponding value.

#' You should use this feature with care in case of \code{case = "parsed"}, \code{case = "internal_parsing"} and 
#' \code{case = "none"}, since for upper case letters, which have transliterations/replacements
#'  of length 2, the second letter will be transliterated to lowercase, for example Oe, Ae, Ss, which
#'  might not always be what is intended. In this case you can make usage of the option to supply named elements and specify the transliterations yourself.
#' 
#' @param case Length one character, from the input options of \code{to_any_case}.
#' 
#' @return A character vector.
#'
#' @author Malte Grosser, \email{malte.grosser@@gmail.com}
#' @keywords utilities
#'
replace_special_characters_internal <- function(string, transliterations, case){
  dictionary <- list(
    german = c("\u00C4" = "Ae", "\u00D6" = "Oe", "\u00DC" = "Ue",
               "\u00E4" = "ae", "\u00F6" = "oe", "\u00FC" = "ue",
               "\u00DF" = "ss"),
    danish = c("\u00C6" = "Ae",
               "\u00E6" = "ae",
               "\u00D8" = "Oe",
               "\u00F8" = "oe",
               "\u00C5" = "Aa",
               "\u00E5" = "aa"),
    finnish = c("\u00C6" = "A",
               "\u00E6" = "a",
               "\u00D8" = "O",
               "\u00F8" = "o"),
    swedish = c("\u00C6" = "A",
                "\u00E6" = "a",
                "\u00D8" = "O",
                "\u00F8" = "o",
                "\u00C5" = "A",
                "\u00E5" = "a")
    )
  
  for (i in seq_along(transliterations)){
    if(isTRUE(!is.null(names(transliterations)[i]) &
              names(transliterations)[i] != "")){
      names(transliterations)[i] <- enc2utf8(names(transliterations))[i]
      
      string <- stringr::str_replace_all(string, transliterations[i])
    } else if(transliterations[i] %in% names(dictionary)){
      
      names(dictionary[[transliterations[i]]]) <- enc2utf8(names(dictionary[[transliterations[i]]]))
        string <- stringr::str_replace_all(enc2utf8(string), dictionary[[transliterations[i]]]) 
      } else if(transliterations[i] %in% stringi::stri_trans_list()){
        string <- stringi::stri_trans_general(string, transliterations[i])
      } else {
          stop("Input to `transliterations` must be `NULL`, a string containing elements from the internal lookup dictionaries or from `stringi::stri_trans_list()` or a named vector.",
                  call. = FALSE)
      }
  }
  # "\u0025" = "_percent_",
    # "\\`" = "",
    # "\\'" = "",
    # "\\@" = "_at_")
  string
}

#' Parsing helpers
#' 
#' Mainly for usage within \code{to_parsed_case_internal}
#'
#' @param string A string.
#'  
#' @return A partly parsed character vector.
#' 
#' @author Malte Grosser, \email{malte.grosser@@gmail.com}
#' 
#' @keywords utilities
#' 
#' @name parsing_helpers
#'
#' @rdname parsing_helpers
#' 
parse1_pat_cap_smalls <- function(string) {
# Inserts underscores around groups of one upper case followed by lower case letters (or digits)
# RStudio.V11 -> R_Studio_.V11
  pat_cap_smalls <- "([:upper:][:lower:]+)"
  string <- stringr::str_replace_all(string, pat_cap_smalls, "_\\1_")
  string}

#' @rdname parsing_helpers
parse2_pat_digits <- function(string) {
# Inserts underscores around groups of digts
# RStudio.V11 -> RStudio.V_11_ 
  pat_digits <- "(\\d+)"
  string <- stringr::str_replace_all(string, pat_digits, "_\\1_")
  string}

#' @rdname parsing_helpers
parse3_pat_caps <- function(string) {
# Inserts underscores around all capital letter groups with length >= 2
# RStudio.V11 -> _RS_tudio.V11
  pat_caps <- "([:upper:]{2,})"
  string <- stringr::str_replace_all(string, pat_caps, "_\\1_")
  string}

#' @rdname parsing_helpers
parse4_pat_cap <- function(string) {
# Inserts underscores around all capital letter groups with length = 1 that
# don't have a capital letter in front of them and not a capital or small letter behind them
# RStudio.V11 -> RStudio._V_11
  pat_cap <- "((?<![:upper:])[:upper:]{1}(?![[:alpha:]]))"
  string <- stringr::str_replace_all(string, pat_cap, "_\\1_")
  string}

#' @rdname parsing_helpers
parse5_pat_non_alnums = function(string){
# Inserts underscores around non-alphanumerics
# RStudio.V11 -> RStudio_._V11
  pat_non_alnums <- "([^[:alnum:]])"
  string <- stringr::str_replace_all(string, pat_non_alnums, "_\\1_")
  string}

#' @rdname parsing_helpers
parse6_mark_digits = function(string) {
# Inserts _ and space between non-alphanumerics and digits
  digit_marker_before <- "(?<=[^_|\\d])(\\d)"
  digit_marker_after  <- "(\\d)(?=[^_|\\d])"
  string <- stringr::str_replace_all(string, digit_marker_before, "_ \\1")
  string <- stringr::str_replace_all(string, digit_marker_after , "\\1 _")
  string
  }

#' @rdname parsing_helpers
parse7_pat_caps_smalls = function(string) {
  # Inserts underscores around of one or more upper case letters possibly followed by lower case letters
  pat_caps_smalls <- "([:upper:]+[:lower:]*)"
  string <- stringr::str_replace_all(string, pat_caps_smalls, "_\\1_")
  string
  }

#' @rdname parsing_helpers
parse8_pat_smalls_after_non_alnums = function(string) {
  # Inserts underscores around of one or more upper case letters possibly followed by lower case letters
  pat_smalls_after_non_alnums <- "((?<![:alnum:])[:lower:]+)"
  string <- stringr::str_replace_all(string, pat_smalls_after_non_alnums, "_\\1_")
  string
  }

---
title: "Introducing the snakecase package"
author: "Malte Grosser"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introducing the snakecase package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE,
                      comment = "#>",
                      collapse = TRUE
                      )
```

There are many style guides out there which recommend specific naming conventions for programming languages. At 2017’s useR conference Rasmus Bååth showed quite impressively the variety of cases which even exist within base R in his talk ["The current state of naming conventions in R"](https://www.youtube.com/watch?v=Pv5dfsHBBKE).

However, consistent style is not only about naming new objects.

## Import

When you do a data analysis, most of the data already exists and you import it from disk, an API or a database. Here is the first moment in your data analysis when you have to decide if you want to rename your data or leave it as it is.

Let’s say you have some data named in any of the following conventions

```{r}
string <- c("lowerCamelCase", "ALL_CAPS", "IDontKNOWWhat_thisCASE_is")
```

You can now easily convert this string for example to snake case via

```{r}
library(snakecase)
to_snake_case(string)
```

## Graphics

Whenever you want to construct a graphic and you don’t like your conventions to come up in it, you can easily convert strings to a more humanly readable output like

```{r}
to_mixed_case(string, sep_out = " ")
```

You might have noticed the `sep_out` argument. This allows you to combine any case with any output separator to create other well known cases like

```{r}
to_snake_case(string, sep_out = ".")
to_snake_case(string, sep_out = "-")
```

or completely new ones like

```{r}
to_screaming_snake_case(string, sep_out = "=")
```

## Export

Finally, when you are done with your analysis and want to write data back into a .CSV file or your customers database, which has a camel case convention, you can just use

```{r}
to_upper_camel_case(string)
```

## Further information

The snakecase package goes quite deep into the little quirks which arise in automatic case conversion. However, it is well tweaked, to handle almost every edge case in an intuitive and elegant manner.

To get a complete overview of its functionality like other cases, handling of abbreviations, special input characters, different parsing options, transliterations and more, I recommend you to have a look into the quite extensive [readme on its github repository](https://github.com/Tazinho/snakecase).

As the package is relatively small and basically consists of its workhorse function `to_any_case()`, I can also react quite fast on new [issues](https://github.com/Tazinho/snakecase/issues).

And of course I [tweet](https://twitter.com/malte_grosser) occasionally about new functionality.

To round this up let me give you one advice about best practices: be aware that automatic case conversion depends on the input string and it is recommended to verify the results. Hence you might want to pipe them into `dput()` and hard-code name changes instead of blindly trusting the output

```{r}
library(magrittr)
to_any_case(c("SomeBAdInput", "someGoodInput")) %>% dput()
```

Happy snakecasing everyone ;)

---
title: "Case converters"
author: "Malte Grosser"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Caseconverters}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## Basic examples

Default case is snake case

```{r, collapse = TRUE}
library(snakecase)
to_any_case("veryComplicatedString")
```

Of course other cases are supported (`case`) and separators can be adjusted (`sep_out`)

```{r, collapse = TRUE}
to_any_case(names(iris), sep_in = "\\.", case = "upper_camel", sep_out = " ")
```

And you might want to remove special characters along the way

```{r, collapse = TRUE}
to_any_case("Doppelgänger is originally german", 
            transliterations = "german", case = "upper_camel")
```

All of the cases like: snake, lower_camel, upper_camel, all_caps, lower_upper, upper_lower, mixed and sentence are based on parsed case

```{r, collapse = TRUE}
to_any_case("THISIsHOW IAmPARSED!", case = "parsed")
```

Shortcut wrappers like `to_snake_case`, `to_lower_camel_case` etc. are available.

Be aware that automatic case conversion depends on the input string and it is recommended to verify the results. So you might want to pipe these into `dput()` and hardcode name changes instead of blindly trusting `to_any_case()`'s output:

```{r, collapse = TRUE}
dput(to_any_case(c("SomeBAdInput", "someGoodInput")))
```

If you are interested in the design of this package, you can find more information on its [github page](https://github.com/Tazinho/snakecase).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parsing_helpers_internal.R
\name{parsing_helpers}
\alias{parsing_helpers}
\alias{parse1_pat_cap_smalls}
\alias{parse2_pat_digits}
\alias{parse3_pat_caps}
\alias{parse4_pat_cap}
\alias{parse5_pat_non_alnums}
\alias{parse6_mark_digits}
\alias{parse7_pat_caps_smalls}
\alias{parse8_pat_smalls_after_non_alnums}
\title{Parsing helpers}
\usage{
parse1_pat_cap_smalls(string)

parse2_pat_digits(string)

parse3_pat_caps(string)

parse4_pat_cap(string)

parse5_pat_non_alnums(string)

parse6_mark_digits(string)

parse7_pat_caps_smalls(string)

parse8_pat_smalls_after_non_alnums(string)
}
\arguments{
\item{string}{A string.}
}
\value{
A partly parsed character vector.
}
\description{
Mainly for usage within \code{to_parsed_case_internal}
}
\author{
Malte Grosser, \email{malte.grosser@gmail.com}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/replace_special_characters_internal.R
\name{replace_special_characters_internal}
\alias{replace_special_characters_internal}
\title{Internal helper to replace special characters.}
\usage{
replace_special_characters_internal(string, transliterations, case)
}
\arguments{
\item{string}{A string (for example names of a data frame).}

\item{transliterations}{A character vector (if not \code{NULL}). The entries of this argument
need to be elements of \code{stringi::stri_trans_list()} (like "Latin-ASCII", which is often useful) or names of lookup tables (currently
only "german" is supported). In the order of the entries the letters of the input
 string will be transliterated via \code{stringi::stri_trans_general()} or replaced via the 
 matches of the lookup table. When named character elements are supplied as part of `transliterations`, anything that matches the names is replaced by the corresponding value.
You should use this feature with care in case of \code{case = "parsed"}, \code{case = "internal_parsing"} and 
\code{case = "none"}, since for upper case letters, which have transliterations/replacements
 of length 2, the second letter will be transliterated to lowercase, for example Oe, Ae, Ss, which
 might not always be what is intended. In this case you can make usage of the option to supply named elements and specify the transliterations yourself.}

\item{case}{Length one character, from the input options of \code{to_any_case}.}
}
\value{
A character vector.
}
\description{
Internal helper to replace special characters.
}
\author{
Malte Grosser, \email{malte.grosser@gmail.com}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/to_any_case.R
\name{to_any_case}
\alias{to_any_case}
\title{General case conversion}
\usage{
to_any_case(string, case = c("snake", "small_camel", "big_camel",
  "screaming_snake", "parsed", "mixed", "lower_upper", "upper_lower",
  "swap", "all_caps", "lower_camel", "upper_camel", "internal_parsing",
  "none", "flip", "sentence", "random", "title"), abbreviations = NULL,
  sep_in = "[^[:alnum:]]", parsing_option = 1,
  transliterations = NULL, numerals = c("middle", "left", "right",
  "asis", "tight"), sep_out = NULL, unique_sep = NULL,
  empty_fill = NULL, prefix = "", postfix = "")
}
\arguments{
\item{string}{A string (for example names of a data frame).}

\item{case}{The desired target case, provided as one of the following:
\itemize{
 \item{snake_case: \code{"snake"}}
 \item{lowerCamel: \code{"lower_camel"} or \code{"small_camel"}}
 \item{UpperCamel: \code{"upper_camel"} or \code{"big_camel"}}
 \item{ALL_CAPS: \code{"all_caps"} or \code{"screaming_snake"}}
 \item{lowerUPPER: \code{"lower_upper"}}
 \item{UPPERlower: \code{"upper_lower"}}
 \item{Sentence case: \code{"sentence"}}
 \item{Title Case: \code{"title"} - This one is basically the same as sentence case, but in addition it is wrapped into \code{tools::toTitleCase} and any \code{abbreviations} are always turned into upper case.}
}

 There are five "special" cases available:
\itemize{
 \item{\code{"parsed"}: This case is underlying all other cases. 
 Every substring a string consists
 of becomes surrounded by an underscore (depending on the \code{parsing_option}).
  Underscores at the start and end are trimmed. No lower or 
 upper case pattern from the input string are changed.}
 \item{\code{"mixed"}: Almost the same as \code{case = "parsed"}. Every letter which is not at the start
 or behind an underscore is turned into lowercase. If a substring is set as an abbreviation, it will be turned into upper case.}
 \item{\code{"swap"}: Upper case letters will be turned into lower case and vice versa. Also \code{case = "flip"} will work.
 Doesn't work with any of the other arguments except \code{unique_sep}, \code{empty_fill}, \code{prefix} and \code{postfix}.}
 \item{\code{"random"}: Each letter will be randomly turned into lower or upper case. Doesn't work with any of the other arguments except \code{unique_sep}, \code{empty_fill}, \code{prefix} and \code{postfix}.}
 \item{\code{"none"}: Neither parsing nor case conversion occur. This case might be helpful, when
 one wants to call the function for the quick usage of the other parameters.
 To suppress replacement of spaces to underscores set \code{sep_in = NULL}.
 Works with \code{sep_in}, \code{transliterations}, \code{sep_out}, \code{prefix},
  \code{postfix},
  \code{empty_fill} and \code{unique_sep}.}
 \item{\code{"internal_parsing"}: This case is returning the internal parsing
 (suppressing the internal protection mechanism), which means that alphanumeric characters will be surrounded by underscores.
 It should only be used in very rare use cases and is mainly implemented to showcase the internal workings of \code{to_any_case()}}
 }}

\item{abbreviations}{character. (Case insensitive) matched abbreviations are surrounded by underscores. In this way, they can get recognized by the parser. This is useful when e.g. \code{parsing_option} 1 is needed for the use case, but some abbreviations but some substrings would require \code{parsing_option} 2. Furthermore, this argument also specifies the formatting of abbreviations in the output for the cases title, mixed, lower and upper camel. E.g. for upper camel the first letter is always in upper case, but when the abbreviation is supplied in upper case, this will also be visible in the output.

Use this feature with care: One letter abbreviations and abbreviations next to each other are hard to read and also not easy to parse for further processing.}

\item{sep_in}{(short for separator input) if character, is interpreted as a
regular expression (wrapped internally into \code{stringr::regex()}). 
The default value is a regular expression that matches any sequence of
non-alphanumeric values. All matches will be replaced by underscores 
(additionally to \code{"_"} and \code{" "}, for which this is always true, even
if \code{NULL} is supplied). These underscores are used internally to split
the strings into substrings and specify the word boundaries.}

\item{parsing_option}{An integer that will determine the parsing_option.
\itemize{
 \item{1: \code{"RRRStudio" -> "RRR_Studio"}}
 \item{2: \code{"RRRStudio" -> "RRRS_tudio"}}
 \item{3: \code{"RRRStudio" -> "RRRSStudio"}. This will become for example \code{"Rrrstudio"} when we convert to lower camel case.}
 \item{-1, -2, -3: These \code{parsing_options}'s will suppress the conversion after non-alphanumeric values.}
 \item{0: no parsing}
 }}

\item{transliterations}{A character vector (if not \code{NULL}). The entries of this argument
need to be elements of \code{stringi::stri_trans_list()} (like "Latin-ASCII", which is often useful) or names of lookup tables (currently only "german" is supported). In the order of the entries the letters of the input
 string will be transliterated via \code{stringi::stri_trans_general()} or replaced via the 
 matches of the lookup table. When named character elements are supplied as part of `transliterations`, anything that matches the names is replaced by the corresponding value.
You should use this feature with care in case of \code{case = "parsed"}, \code{case = "internal_parsing"} and 
\code{case = "none"}, since for upper case letters, which have transliterations/replacements
 of length 2, the second letter will be transliterated to lowercase, for example Oe, Ae, Ss, which
 might not always be what is intended. In this case you can make usage of the option to supply named elements and specify the transliterations yourself.}

\item{numerals}{A character specifying the alignment of numerals (\code{"middle"}, \code{left}, \code{right}, \code{asis} or \code{tight}). I.e. \code{numerals = "left"} ensures that no output separator is in front of a digit.}

\item{sep_out}{(short for separator output) String that will be used as separator. The defaults are \code{"_"} 
and \code{""}, regarding the specified \code{case}. When \code{length(sep_out) > 1}, the last element of \code{sep_out} gets recycled and separators are incorporated per string according to their order.}

\item{unique_sep}{A string. If not \code{NULL}, then duplicated names will get 
a suffix integer
in the order of their appearance. The suffix is separated by the supplied string
 to this argument.}

\item{empty_fill}{A string. If it is supplied, then each entry that matches "" will be replaced
by the supplied string to this argument.}

\item{prefix}{prefix (string).}

\item{postfix}{postfix (string).}
}
\value{
A character vector according the specified parameters above.
}
\description{
Function to convert strings to any case
}
\note{
\code{to_any_case()} is vectorised over \code{string}, \code{sep_in}, \code{sep_out},
 \code{empty_fill}, \code{prefix} and \code{postfix}.
}
\examples{
### abbreviations
to_snake_case(c("HHcity", "newUSElections"), abbreviations = c("HH", "US"))
to_upper_camel_case("succesfullGMBH", abbreviations = "GmbH")
to_title_case("succesfullGMBH", abbreviations = "GmbH")

### sep_in (input separator)
string <- "R.St\\u00FCdio: v.1.0.143"
to_any_case(string)
to_any_case(string, sep_in = ":|\\\\.")
to_any_case(string, sep_in = ":|(?<!\\\\d)\\\\.")
            
### parsing_option
# the default option makes no sense in this setting
to_parsed_case("HAMBURGcity", parsing_option = 1)
# so the second parsing option is the way to address this example
to_parsed_case("HAMBURGcity", parsing_option = 2)
# By default (option 1) characters are converted after non alpha numeric characters.
# To suppress this behaviour add a minus to the parsing_option
to_upper_camel_case("lookBehindThe.dot", parsing_option = -1)
# For some exotic cases parsing option 3 might be of interest
to_parsed_case("PARSingOption3", parsing_option = 3)
# There may be reasons to suppress the parsing
to_any_case("HAMBURGcity", parsing_option = 0)

### transliterations
to_any_case("\\u00E4ngstlicher Has\\u00EA", transliterations = c("german", "Latin-ASCII"))

### case
strings <- c("this Is a Strange_string", "AND THIS ANOTHER_One")
to_any_case(strings, case = "snake")
to_any_case(strings, case = "lower_camel") # same as "small_camel"
to_any_case(strings, case = "upper_camel") # same as "big_camel"
to_any_case(strings, case = "all_caps") # same as "screaming_snake"
to_any_case(strings, case = "lower_upper")
to_any_case(strings, case = "upper_lower")
to_any_case(strings, case = "sentence")
to_any_case(strings, case = "title")
to_any_case(strings, case = "parsed")
to_any_case(strings, case = "mixed")
to_any_case(strings, case = "swap")
to_any_case(strings, case = "random")
to_any_case(strings, case = "none")
to_any_case(strings, case = "internal_parsing")

### numerals
to_snake_case("species42value 23month 7-8", numerals = "asis")
to_snake_case("species42value 23month 7-8", numerals = "left")
to_snake_case("species42value 23month 7-8", numerals = "right")
to_snake_case("species42value 23month 7-8", numerals = "middle")
to_snake_case("species42value 23month 7-8", numerals = "tight")

### sep_out (output separator)
string <- c("lowerCamelCase", "ALL_CAPS", "I-DontKNOWWhat_thisCASE_is")
to_snake_case(string, sep_out = ".")
to_mixed_case(string, sep_out = " ")
to_screaming_snake_case(string, sep_out = "=")

### empty_fill
to_any_case(c("","",""), empty_fill = c("empty", "empty", "also empty"))

### unique_sep
to_any_case(c("same", "same", "same", "other"), unique_sep = c(">"))

### prefix and postfix
to_upper_camel_case("some_path", sep_out = "//", 
  prefix = "USER://", postfix = ".exe")

}
\seealso{
\href{https://github.com/Tazinho/snakecase}{snakecase on github} or 
\code{\link{caseconverter}} for some handy shortcuts.
}
\author{
Malte Grosser, \email{malte.grosser@gmail.com}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/to_parsed_case_internal.R
\name{to_parsed_case_internal}
\alias{to_parsed_case_internal}
\title{Internal parser, which is relevant for preprocessing, parsing and parsing options}
\usage{
to_parsed_case_internal(string, parsing_option = 1L, numerals,
  abbreviations, sep_in)
}
\arguments{
\item{string}{A string.}

\item{parsing_option}{An integer that will determine the parsing option.
\itemize{
 \item{1: \code{RRRStudio -> RRR_Studio}}
 \item{2: \code{RRRStudio -> RRRS_tudio}}
 \item{3: parses like option 1 but suppresses "_" around non alpha-numeric characters. In this way this option suppresses splits and resulting case conversion after these characters.}
 \item{any other integer <= 0: no parsing"}
 }}

\item{numerals}{A character specifying the alignment of numerals (\code{"middle"}, \code{left}, \code{right} or \code{asis}). I.e. \code{numerals = "left"} ensures that no output separator is in front of a digit.}

\item{abbreviations}{A character string specifying abbreviations that should be marked to be recognized by later parsing.}

\item{sep_in}{A character (regular expression) used to specify input separators.}
}
\value{
A character vector separated by underscores, containing the parsed string.
}
\description{
Internal parser, which is relevant for preprocessing, parsing and parsing options
}
\author{
Malte Grosser, \email{malte.grosser@gmail.com}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/caseconverters.R
\name{caseconverter}
\alias{caseconverter}
\alias{to_snake_case}
\alias{to_lower_camel_case}
\alias{to_upper_camel_case}
\alias{to_screaming_snake_case}
\alias{to_parsed_case}
\alias{to_mixed_case}
\alias{to_lower_upper_case}
\alias{to_upper_lower_case}
\alias{to_swap_case}
\alias{to_sentence_case}
\alias{to_random_case}
\alias{to_title_case}
\title{Specific case converter shortcuts}
\usage{
to_snake_case(string, abbreviations = NULL, sep_in = "[^[:alnum:]]",
  parsing_option = 1, transliterations = NULL, numerals = "middle",
  sep_out = NULL, unique_sep = NULL, empty_fill = NULL,
  prefix = "", postfix = "")

to_lower_camel_case(string, abbreviations = NULL,
  sep_in = "[^[:alnum:]]", parsing_option = 1,
  transliterations = NULL, numerals = "middle", sep_out = NULL,
  unique_sep = NULL, empty_fill = NULL, prefix = "", postfix = "")

to_upper_camel_case(string, abbreviations = NULL,
  sep_in = "[^[:alnum:]]", parsing_option = 1,
  transliterations = NULL, numerals = "middle", sep_out = NULL,
  unique_sep = NULL, empty_fill = NULL, prefix = "", postfix = "")

to_screaming_snake_case(string, abbreviations = NULL,
  sep_in = "[^[:alnum:]]", parsing_option = 1,
  transliterations = NULL, numerals = "middle", sep_out = NULL,
  unique_sep = NULL, empty_fill = NULL, prefix = "", postfix = "")

to_parsed_case(string, abbreviations = NULL, sep_in = "[^[:alnum:]]",
  parsing_option = 1, transliterations = NULL, numerals = "middle",
  sep_out = NULL, unique_sep = NULL, empty_fill = NULL,
  prefix = "", postfix = "")

to_mixed_case(string, abbreviations = NULL, sep_in = "[^[:alnum:]]",
  parsing_option = 1, transliterations = NULL, numerals = "middle",
  sep_out = NULL, unique_sep = NULL, empty_fill = NULL,
  prefix = "", postfix = "")

to_lower_upper_case(string, abbreviations = NULL,
  sep_in = "[^[:alnum:]]", parsing_option = 1,
  transliterations = NULL, numerals = "middle", sep_out = NULL,
  unique_sep = NULL, empty_fill = NULL, prefix = "", postfix = "")

to_upper_lower_case(string, abbreviations = NULL,
  sep_in = "[^[:alnum:]]", parsing_option = 1,
  transliterations = NULL, numerals = "middle", sep_out = NULL,
  unique_sep = NULL, empty_fill = NULL, prefix = "", postfix = "")

to_swap_case(string, abbreviations = NULL, sep_in = "[^[:alnum:]]",
  parsing_option = 1, transliterations = NULL, numerals = "middle",
  sep_out = NULL, unique_sep = NULL, empty_fill = NULL,
  prefix = "", postfix = "")

to_sentence_case(string, abbreviations = NULL, sep_in = "[^[:alnum:]]",
  parsing_option = 1, transliterations = NULL, numerals = "middle",
  sep_out = NULL, unique_sep = NULL, empty_fill = NULL,
  prefix = "", postfix = "")

to_random_case(string, abbreviations = NULL, sep_in = "[^[:alnum:]]",
  parsing_option = 1, transliterations = NULL, numerals = "middle",
  sep_out = NULL, unique_sep = NULL, empty_fill = NULL,
  prefix = "", postfix = "")

to_title_case(string, abbreviations = NULL, sep_in = "[^[:alnum:]]",
  parsing_option = 1, transliterations = NULL, numerals = "middle",
  sep_out = NULL, unique_sep = NULL, empty_fill = NULL,
  prefix = "", postfix = "")
}
\arguments{
\item{string}{A string (for example names of a data frame).}

\item{abbreviations}{character. (Case insensitive) matched abbreviations are surrounded by underscores. In this way, they can get recognized by the parser. This is useful when e.g. \code{parsing_option} 1 is needed for the use case, but some abbreviations but some substrings would require \code{parsing_option} 2. Furthermore, this argument also specifies the formatting of abbreviations in the output for the cases title, mixed, lower and upper camel. E.g. for upper camel the first letter is always in upper case, but when the abbreviation is supplied in upper case, this will also be visible in the output.

Use this feature with care: One letter abbreviations and abbreviations next to each other are hard to read and also not easy to parse for further processing.}

\item{sep_in}{(short for separator input) if character, is interpreted as a
regular expression (wrapped internally into \code{stringr::regex()}). 
The default value is a regular expression that matches any sequence of
non-alphanumeric values. All matches will be replaced by underscores 
(additionally to \code{"_"} and \code{" "}, for which this is always true, even
if \code{NULL} is supplied). These underscores are used internally to split
the strings into substrings and specify the word boundaries.}

\item{parsing_option}{An integer that will determine the parsing_option.
\itemize{
 \item{1: \code{"RRRStudio" -> "RRR_Studio"}}
 \item{2: \code{"RRRStudio" -> "RRRS_tudio"}}
 \item{3: \code{"RRRStudio" -> "RRRSStudio"}. This will become for example \code{"Rrrstudio"} when we convert to lower camel case.}
 \item{-1, -2, -3: These \code{parsing_options}'s will suppress the conversion after non-alphanumeric values.}
 \item{0: no parsing}
 }}

\item{transliterations}{A character vector (if not \code{NULL}). The entries of this argument
need to be elements of \code{stringi::stri_trans_list()} (like "Latin-ASCII", which is often useful) or names of lookup tables (currently
only "german" is supported). In the order of the entries the letters of the input
 string will be transliterated via \code{stringi::stri_trans_general()} or replaced via the 
 matches of the lookup table. When named character elements are supplied as part of `transliterations`, anything that matches the names is replaced by the corresponding value.
You should use this feature with care in case of \code{case = "parsed"}, \code{case = "internal_parsing"} and 
\code{case = "none"}, since for upper case letters, which have transliterations/replacements
 of length 2, the second letter will be transliterated to lowercase, for example Oe, Ae, Ss, which
 might not always be what is intended. In this case you can make usage of the option to supply named elements and specify the transliterations yourself.}

\item{numerals}{A character specifying the alignment of numerals (\code{"middle"}, \code{left}, \code{right} or \code{asis}). I.e. \code{numerals = "left"} ensures that no output separator is in front of a digit.}

\item{sep_out}{(short for separator output) String that will be used as separator. The defaults are \code{"_"} 
and \code{""}, regarding the specified \code{case}. When \code{length(sep_out) > 1}, the last element of \code{sep_out} gets recycled and separators are incorporated per string according to their order.}

\item{unique_sep}{A string. If not \code{NULL}, then duplicated names will get 
a suffix integer
in the order of their appearance. The suffix is separated by the supplied string
 to this argument.}

\item{empty_fill}{A string. If it is supplied, then each entry that matches "" will be replaced
by the supplied string to this argument.}

\item{prefix}{prefix (string).}

\item{postfix}{postfix (string).}
}
\value{
A character vector according the specified parameters above.

A character vector according the specified target case.
}
\description{
Wrappers around \code{to_any_case()}
}
\note{
caseconverters are vectorised over \code{string}, \code{sep_in}, \code{sep_out},
 \code{empty_fill}, \code{prefix} and \code{postfix}.
}
\examples{
strings <- c("this Is a Strange_string", "AND THIS ANOTHER_One", NA)

to_snake_case(strings)
to_lower_camel_case(strings)
to_upper_camel_case(strings)
to_screaming_snake_case(strings)
to_lower_upper_case(strings)
to_upper_lower_case(strings)
to_parsed_case(strings)
to_mixed_case(strings)
to_swap_case(strings)
to_sentence_case(strings)
to_random_case(strings)
to_title_case(strings)


}
\seealso{
\href{https://github.com/Tazinho/snakecase}{snakecase on github}, \code{\link{to_any_case}} for flexible high level conversion and more examples.
}
\author{
Malte Grosser, \email{malte.grosser@gmail.com}

Malte Grosser, \email{malte.grosser@gmail.com}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_design_rule.R
\name{check_design_rule}
\alias{check_design_rule}
\title{Internal helper to test the design rules for any string and setting of \code{to_any_case()}}
\usage{
check_design_rule(string, sep_in = NULL, transliterations = NULL,
  sep_out = NULL, prefix = "", postfix = "", unique_sep = NULL,
  empty_fill = NULL, parsing_option = 1)
}
\arguments{
\item{string}{A string (for example names of a data frame).}

\item{sep_in}{String that will be wrapped internally into \code{stringr::regex()}. 
All matches will be treated as additional splitting parameters besides the default ones 
(\code{"_"} and \code{" "}), when parsing the input string.}

\item{transliterations}{A character vector (if not \code{NULL}). The entries of this argument
need to be elements of \code{stringi::stri_trans_list()} (like "Latin-ASCII", which is often useful) or names of lookup tables (currently
only "german" is supported). In the order of the entries the letters of the input
 string will be transliterated via \code{stringi::stri_trans_general()} or replaced via the 
 matches of the lookup table.}

\item{sep_out}{String that will be used as separator. The defaults are \code{"_"} 
and \code{""}, regarding the specified \code{case}.}

\item{prefix}{prefix (string).}

\item{postfix}{postfix (string).}

\item{unique_sep}{A string. If it is supplied, then duplicated names will get a suffix integer
in the order of their appearance. The suffix is separated by the supplied string to this argument.}

\item{empty_fill}{A string. If it is supplied, then each entry that matches "" will be replaced
by the supplied string to this argument.}

\item{parsing_option}{An integer that will determine the parsing_option.
\itemize{
 \item{1: \code{RRRStudio -> RRR_Studio}}
 \item{2: \code{RRRStudio -> RRRS_tudio}}
 \item{3: parses at the beginning like option 1 and the rest like option 2.}
 \item{4: parses at the beginning like option 2 and the rest like option 1.}
 \item{5: parses like option 1 but suppresses "_" around non special characters.
 In this way case conversion won't apply after these characters. See examples.}
 \item{6: parses like option 1, but digits directly behind/in front non-digits, will stay as is.}
 \item{any other integer <= 0: no parsing"}
 }}
}
\value{
A character vector separated by underscores, containing the parsed string.
}
\description{
Internal helper to test the design rules for any string and setting of \code{to_any_case()}
}
\author{
Malte Grosser, \email{malte.grosser@gmail.com}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocess_internal.R
\name{preprocess_internal}
\alias{preprocess_internal}
\title{Internal function that replaces regex matches with underscores}
\usage{
preprocess_internal(string, sep_in)
}
\arguments{
\item{string}{A string.}

\item{sep_in}{(short for separator input) A regex supplied as a character (if not \code{NULL}), which will be wrapped internally
into \code{stringr::regex()}. All matches will be replaced by underscores (additionally to 
\code{"_"} and \code{" "}, for which this is always true). Underscores can later turned into another separator via \code{postprocess}.}
}
\value{
A character containing the parsed string.
}
\description{
Internal function that replaces regex matches with underscores
}
\author{
Malte Grosser, \email{malte.grosser@gmail.com}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relevant.R
\name{relevant}
\alias{relevant}
\title{Internal helper for "lower_upper", "upper_lower".
This helper returns a logical vector with TRUE for the 
first and every second string of those which contain 
an alphabetic character}
\usage{
relevant(string)
}
\arguments{
\item{string}{A string (for example names of a data frame).}
}
\value{
A logical vector.
}
\description{
Internal helper for "lower_upper", "upper_lower".
This helper returns a logical vector with TRUE for the 
first and every second string of those which contain 
an alphabetic character
}
\author{
Malte Grosser, \email{malte.grosser@gmail.com}
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/abbreviation_internal.R
\name{abbreviation_internal}
\alias{abbreviation_internal}
\title{Internal abbreviation marker, marks abbreviations with an underscore behind.
Useful if \code{parsing_option} 1 is needed, but some abbreviations need \code{parsing_option} 2.}
\usage{
abbreviation_internal(string, abbreviations = NULL)
}
\arguments{
\item{string}{A string (for example names of a data frame).}

\item{abbreviations}{character with (uppercase) abbreviations. This marks
abbreviations with an underscore behind (in front of the parsing).
Useful if \code{parsing_option} 1 is needed, but some abbreviations need \code{parsing_option} 2.}
}
\value{
A character vector.
}
\description{
Internal abbreviation marker, marks abbreviations with an underscore behind.
Useful if \code{parsing_option} 1 is needed, but some abbreviations need \code{parsing_option} 2.
}
\author{
Malte Grosser, \email{malte.grosser@gmail.com}
}
\keyword{utilities}
